//*************************************************************************
//* -----------------
//* MyPrimaryParticle
//* -----------------
//* (Description)
//* 	Class to cross-reference primary particles and break points.
//* (Update Record)
//*	2007/08/21  K.Fujii	Original version.
//*************************************************************************

#include "MyPrimaryParticle.h"
#include "S4BreakPoint.h"
#include <iostream>

std::vector<MyPrimaryParticle*> MyPrimaryParticle::fgPPList;
std::map<Int_t, MyPrimaryParticle*> MyPrimaryParticle::fgPPMap;


ClassImp(MyPrimaryParticle)

MyPrimaryParticle::MyPrimaryParticle(S4PrimaryParticle *ppp) : S4PrimaryParticle(*ppp), fParentPtr(0)
{
   fDaughters = new TObjArray();

   ppp->SetBit(kUsed);
   fgPPList.push_back(this);
   TObjArray *daup = ppp->GetDaughters();

   if (daup && daup->GetEntries()) {
      for (Int_t i=0; i<daup->GetEntries(); i++) {
         S4PrimaryParticle *s4ppp = static_cast<S4PrimaryParticle *>((*daup)[i]);
         if (!s4ppp->TestBit(kUsed)) {
            MyPrimaryParticle *myppp = new MyPrimaryParticle(s4ppp);
            (*fDaughters)[i] = myppp;
	    myppp->fParentPtr = this;
	 }
      }
   }
}

MyPrimaryParticle::~MyPrimaryParticle()
{
}

MyPrimaryParticle *MyPrimaryParticle::GetMyPPPtr(Int_t j4tkid)
{
   std::map<Int_t, MyPrimaryParticle *>::iterator iter = fgPPMap.find(j4tkid);
   if (iter != fgPPMap.end()) {
      return iter->second;
   } else {
      return 0;
   }
}

MyPrimaryParticle *MyPrimaryParticle::GetGrandParentPtr()
{
   if (fParentPtr) return fParentPtr->GetGrandParentPtr() ? fParentPtr->GetGrandParentPtr() : fParentPtr; 
   return 0;
}

void MyPrimaryParticle::Initialize(TObjArray *pplist)
{
   if (!pplist) return;
   
   std::vector<MyPrimaryParticle *>::iterator iter;
   for (iter = fgPPList.begin(); iter != fgPPList.end(); iter++) {
      delete (*iter);
   }
   fgPPList.clear();

   TIter next(pplist);
   S4PrimaryParticle *s4ppp;
   while ((s4ppp = static_cast<S4PrimaryParticle *>(next()))) {
      if (!s4ppp->TestBit(kUsed)) {
         new MyPrimaryParticle(s4ppp);
      }
   }

   fgPPMap.clear();
   for (iter = fgPPList.begin(); iter != fgPPList.end(); iter++) {
      MyPrimaryParticle *myppp = *iter;
      if (myppp->GetTrackID() > 0) {
         fgPPMap.insert(std::pair<Int_t, MyPrimaryParticle*>(myppp->GetTrackID(), myppp));
      }
   }
}

void MyPrimaryParticle::DebugPrint()
{
   std::vector<MyPrimaryParticle *>::iterator iter;
   for (iter = fgPPList.begin(); iter != fgPPList.end(); iter++) {
      (*iter)->Print();
      std::cerr << " this = "   << (void *)(*iter) 
	        << " parent = " << (void *)(*iter)->GetParentPtr()
	        << " gparent = " << (void *)(*iter)->GetGrandParentPtr()
		<< std::endl;
   }
}

