// $Id: S4CALAddress.h,v 1.7 2007/01/16 19:15:02 ono Exp $
//********************************************************************
//* ---------------
//*  S4CALAddress
//* ---------------
//*   Utility for converting cell address and position.
//*
//* (Update Record)
//*   2004/10/16  Allister Sanchez  Original (preliminary?) version.
//********************************************************************
#ifndef __S4CALAddress__
#define __S4CALAddress__

#include "TVAddress.h"
#include <vector>

class S4CALAddress : public TVAddress {

private:

  static Bool_t   fgIsCALInitialized;
  static Bool_t   fgIsCLXInitialized;

  static Bool_t   gfIsTowerData;
  
  static Bool_t   fgIsCAL;
  static Double_t fgCALTowerFront;
  static Int_t    fgCALNTowers;
  static Int_t    fgCALNIsEM;
  static Int_t    fgCALNBlocks;

  static Int_t    fgCALNEMMiniCones;
  static Int_t    fgCALNEMMiniTowers;
  static Int_t    fgCALNEMLayers;
  static Int_t    fgCALNEMSubLayers;
  static Int_t    fgCALNEMCells;
  static Double_t fgCALEMAbsLayer;
  static Double_t fgCALEMActiveLayer;
  static Double_t fgCALEMLayer;

  static Int_t    fgCALNHDMiniCones;
  static Int_t    fgCALNHDMiniTowers;
  static Int_t    fgCALNHDLayers;
  static Int_t    fgCALNHDSubLayers;
  static Int_t    fgCALNHDCells;
  static Double_t fgCALHDAbsLayer;
  static Double_t fgCALHDActiveLayer;
  static Double_t fgCALHDLayer;

  static std::vector<Int_t>    fgCALNPhi;
  static std::vector<Double_t> fgCALRadius;
  static std::vector<Double_t> fgCALTheta;
  static std::vector<Double_t> fgCALDTheta;

  //* CLX data members
  static Bool_t   fgIsCLX              ;
  static Double_t fgCLXBarrelInnerR    ;
  static Double_t fgCLXEndcapInnerR    ;
  static Double_t fgCLXBarrelHalfZ     ;
  static Double_t fgCLXGap             ;
  static Double_t fgCLXEMActiveThick   ;
  static Double_t fgCLXHDActiveThick   ;
  static Double_t fgCLXEMAbsThick      ;
  static Double_t fgCLXHDAbsThick      ;
  static Double_t fgCLXEMFlexThick     ;
  static Double_t fgCLXHDFlexThick     ;
  static Double_t fgCLXCellSize        ;
  static Double_t fgCLXEMLayerThick    ;
  static Double_t fgCLXHDLayerThick    ;
  static Double_t fgCLXEMThick         ;
  static Double_t fgCLXHDThick         ;
  static Double_t fgCLXEMYmax          ;
  static Double_t fgCLXHDYmax          ;
  static Double_t fgCLXEMHalfZ         ;
  static Double_t fgCLXHDHalfZ         ;
  static Double_t fgCLXEMHeight        ;
  static Double_t fgCLXHDHeight        ;

  static Int_t    fgCLXNTraps          ;
  static Int_t    fgCLXNBlocks         ;
  static Int_t    fgCLXNEMLayers       ;
  static Int_t    fgCLXNHDLayers       ;
  static Int_t    fgCLXNEMTrapStrips   ;
  static Int_t    fgCLXNHDTrapStrips   ;
  static Int_t    fgCLXNEMStrips       ;
  static Int_t    fgCLXNHDStrips       ;
  static Int_t    fgCLXNEMCells        ;
  static Int_t    fgCLXNHDCells        ;
  static Int_t    fgCLXNEMTotals       ;

public:
  S4CALAddress();
  virtual ~S4CALAddress();

  //* Parameter setter for CAL/CLX data
  static  void    SetCALParameters();
  static  void    SetCLXParameters();

  //* CAL tower parameters
  static  void    SetTowerParameters(TString geom);
  static  void    SetTowerParameters();

  //  virtual TVector3 GetAddressPosition(Int_t cellID, Int_t miniconeID, 
  //                      Int_t minitowerID, Int_t layerID, Bool_t isEM);
  //static TVector3 GetAddressPosition(Int_t coneID, Int_t towerID, Int_t miniconeID, 
  //                                   Int_t minitowerID, Int_t layerID, Bool_t isEM);
  //* CAL CellID to Cell position converter
  //  static TVector3 GetAddressPosition(Int_t celLID, Bool_t isEM );

  //* CLX CellID to Cell position converter
  static TVector3 GetAddressPosition( Int_t glocalCellID, Bool_t isEM, Bool_t isBarrel = kTRUE );
  
  virtual Double_t GetDistanceFrom(const TVAddress *addr);
  virtual Double_t GetDistanceFrom(const TVector3 position);

  //* CAL Cell ID
  static Int_t GetCellID(Int_t coneid, Int_t towerid,
			 Int_t miniconeid, Int_t minitowerid,
			 Int_t layerid, Int_t sublayerid, Bool_t isEM);
  //* CLX Cell ID
  static Int_t GetCellID( Bool_t isEM, Int_t  blockID, Int_t  layerID,
			  Int_t trapStripID, Int_t  stripID, Int_t cellID );  

  //* CAL/CLX layerID switched by fIsXXX
  static Int_t  GetLayerID(Int_t cellid, Bool_t isEM);

  //* CAL ID converter
  static Int_t  GetConeID(Int_t cellid);
  static Int_t  GetTowerID(Int_t cellid);
  static Int_t  GetMiniConeID(Int_t cellid, Bool_t isEM);
  static Int_t  GetMiniTowerID(Int_t cellid, Bool_t isEM);
  static Int_t  GetSubLayerID(Int_t cellid, Bool_t isEM);
  static Int_t  GetCALLayerID(Int_t cellid, Bool_t isEM);

  //* CLX ID Converter
  static Int_t  GetCLXBlockID     ( Int_t cellID );
  static Int_t  GetCLXLayerID     ( Int_t cellID, Bool_t isEM );
  static Int_t  GetCLXTrapStripID ( Int_t cellID, Bool_t isEM );  
  static Int_t  GetCLXStripID     ( Int_t cellID, Bool_t isEM );
  static Int_t  GetCLXCellID      ( Int_t cellID, Bool_t isEM );
  static Int_t  IsEndcap          ( Int_t cellID, Bool_t isBarrel = kFALSE );
  static Int_t  IsBarrel          ( Int_t cellID );
  static Bool_t IsBoundary        ( Int_t cellID, Bool_t isEM );
  static Bool_t IsXNegative       ( Int_t cellID, Bool_t isEM, Bool_t isBarrel = kTRUE );  

  static Int_t  GetNCells         ( Int_t trapLayerID, Bool_t isEM, Bool_t isBarrel = kTRUE );
  static Int_t  GetLocalCellID    ( Int_t cellID, Bool_t isEM, Bool_t isBarrel = kTRUE );
  static Int_t  GetYAlignCellID   ( Int_t localID, Int_t trapLayerID, 
                                    Bool_t isEM, Bool_t isBarrel, Bool_t isXNegative );
  static Int_t  GetFrontCellID    ( Int_t localID, Int_t trapLayerID,
				    Bool_t isEM, Bool_t isBarrel, Bool_t isXNegative );
  static Int_t  GetBackCellID     ( Int_t localID, Int_t trapLayerID,
				    Bool_t isEM, Bool_t isBarrel, Bool_t isXNegative );  
  
  //* Calorimeter detector parameters
  static Double_t GetBarrelInnerR();
  static Double_t GetEndcapInnerR();
  static Double_t GetBarrelHalfZ();
  static Double_t GetGap();
  static Double_t GetEMLayerThickness();
  static Double_t GetHDLayerThickness();
  static Double_t GetEMActiveLayerThickness();
  static Double_t GetHDActiveLayerThickness();
  static Double_t GetEMAbsLayerThickness();
  static Double_t GetHDAbsLayerThickness();
  static Int_t    GetEMNLayers();
  static Int_t    GetHDNLayers();

  static Double_t GetEMYMax();
  static Double_t GetHDYMax();

  static void Print( Int_t cellID, Bool_t isEM );
  
  //* List of Neighbor Cell IDs
  static std::vector<Int_t> GetListOfNeighbors( Int_t cellID, Bool_t isEM, Bool_t isBarrel = kTRUE, Int_t nCells = -1 );

  //* CLX all IDs vector
  static std::vector<Int_t> GetCLXIDVector( Int_t cellID, Bool_t isEM );

  //* CAL tower depend informations
  static Int_t    GetNMiniCones(Bool_t isEM);
  static Int_t    GetNMiniTowers(Bool_t isEM);
  static Bool_t   IsTowerData() { if(!fgIsCALInitialized); return  gfIsTowerData; }

  ClassDef(S4CALAddress,2) // S4CALAddress class
};

#endif
