//*************************************************************************
//* ----------------
//* S4CALMCTruth Class
//* ----------------
//* (Description)
//* (Requires)
//*
//* (Provides)
//*	S4CALMCTruthBuf		: raw data buffer
//*	S4CALMCTruth		: steering class
//* (Update Record)
//*	2004/11/24  S.Yamamoto	Original version.
//*************************************************************************

#include "S4CALMCTruth.h"
#include "S4CALExactHit.h"
#include "S4CALMCTruthBuf.h"
#include <iostream>
//#define __DEBUG__
using namespace std;

ClassImp(S4CALMCTruth)

//=====================================================================
//* constructor -------------------------------------------------------

S4CALMCTruth::S4CALMCTruth(const Char_t *fname, 
                           const Char_t *name,
                           const Char_t *title)
            : S4VMCTruth(name, title),
              fInputFileName((Char_t *)fname)
{
#ifdef __DEBUG__
   cerr << " constructor for S4CALMCTruth is called..." << endl;
#endif
   fEventBuf = new S4CALMCTruthBuf(this);
}

//=====================================================================
//* destructor --------------------------------------------------------

S4CALMCTruth::~S4CALMCTruth()
{
#ifdef __DEBUG__
   cerr << " S4CALMCTruth will be deleted...fEventBuf is " 
        << (Int_t)fEventBuf << endl;
#endif
   if(fEventBuf) delete fEventBuf;
   if(!fInputStream) fInputStream->close();
}

//=====================================================================
//* BeginRun  --------------------------------------------------------

Bool_t S4CALMCTruth::BeginRun(Int_t nrun)
{
#ifdef __DEBUG__
   cerr << "S4CALMCTruth::BeginRun() is called..." << endl;
#endif
    return S4VMCTruth::BeginRun(nrun);

}

//=====================================================================
//* Process   --------------------------------------------------------

Bool_t S4CALMCTruth::Process(Int_t ev)
{
   S4VMCTruth::Process(ev);
#ifdef __DEBUG__
   cerr << "S4CAL`MCTruth::Process() is called..." << endl;  
#endif
   S4CALMCTruthBuf *bufp = (S4CALMCTruthBuf *)fEventBuf;

   if (fFile->IsWritable()) { // JSFJ4
      if (bufp->GetInputBufPtr()) {
         TObjArray *ehitbufp   = bufp->GetExactHitsPtr(); 
         ehitbufp->Clear();
         TIter next(bufp->GetInputBufPtr());
         TObject *objp = 0;
         while ((objp = next())) ehitbufp->Add(objp);
#ifdef __DEBUG__
         cerr << " CALHitsPtr = " << (void *)bufp->GetInputBufPtr() << endl;
         cerr << " Nhits      = " << bufp->GetInputBufPtr()->GetEntries() << endl;
#endif
      } else { // Read ASCII data
         bufp->GetExactHitsPtr()->Delete();
         bufp->GetPreHitsPtr()->Delete();
         bufp->ReadFile();
      } 
      bufp->UpdatePtrs();
#ifdef __DEBUG__
      cerr << "S4CALMCTruth::Process nhits =" 
           << bufp->GetExactHitsPtr()->GetEntries() << endl;
#if 0
      DebugPrint();
#else
      Print();
#endif
#endif
      return kTRUE;
   }
   else {
      cerr << "S4CALMCTruth::Process ExacthitsBuffer is not updated. " << endl;
      return kTRUE;
   }
}


//=====================================================================
//* DebugPrint -------------------------------------------------------

void S4CALMCTruth::DebugPrint(const Char_t *)
{
   S4CALMCTruthBuf *bufp = (S4CALMCTruthBuf *)fEventBuf;
   TIter next(bufp->GetExactHitsPtr()); 
   S4CALExactHit *ehitp;
   while ((ehitp = static_cast<S4CALExactHit *>(next()))) {
      ehitp->DebugPrint();
   }
}
