#ifndef __S4CALPreHit__
#define __S4CALPreHit__

//////////////////////////////////////////////////////////////////////
//
//  JSFJ4 CAL PreHit Data
//
//  2004/10/16  Allister Sanchez   Original version.
//
//  $Id: S4CALPreHit.h,v 1.5 2006/02/10 14:21:28 miyamoto Exp $
/////////////////////////////////////////////////////////////////////

#include "S4VExactHit.h"
#include "S4TPCPostHit.h"
#include <TVector3.h>
#include <TString.h>

class J4CALPreHit;
class J4CLXPreHit;

// ***************************************************************
class S4CALPreHit : public S4VExactHit
{
 public:
  S4CALPreHit();
  S4CALPreHit(Int_t      postHitID,      // posthit ID
	      Int_t      preHitID,       // prehit ID
	      Int_t      trackID,        // track ID 
	      Int_t      motherTrackID,  // mother track ID
	      Int_t      pcode,          // PDG code
	      Double_t   charge,         // charge
	      TVector3   pos,            // Injection position [cm]
	      Double_t   tof,            // TOF      [nsec]
	      TVector3   mom,            // momentum [GeV] 
	      Double_t   etot,           // energy
	      Double_t   edep,           // energy deposit [GeV]      
	      TVAddress *p = 0);         // address ptr
  S4CALPreHit(J4CALPreHit *prehit);
  S4CALPreHit(J4CLXPreHit *prehit);
  virtual ~S4CALPreHit();

  static void   SetSortBy (Char_t *opt="");
         Int_t  Compare   (const TObject*) const;
         Bool_t IsSortable()               const;

  inline Int_t         GetPostHitID     ()  const { return fPostHitID;       }
  inline Int_t         GetPreHitID      ()  const { return GetHitID();       }
  inline Double_t      GetEnergy        ()  const { return GetTotalEnergy(); }
  inline TVector3      GetInjectionPoint()  const { return GetPosition();    }
  inline S4TPCPostHit* GetPostHitPtr()      const { return fPostHitPtr;      }
  inline void          SetPostHitPtr(S4TPCPostHit *p){fPostHitPtr = p; }

  void Print(Option_t *opt="") const;

 private:
  TString       fParticleName;  //  Particle Name
  Int_t         fPostHitID;     //  post hit id
  S4TPCPostHit *fPostHitPtr;    //! post hit ptr

  ClassDef(S4CALPreHit, 2) // S4CALPreHit class
};

#endif
