#ifndef __S4CDCEXACTHIT__
#define __S4CDCEXACTHIT__
//*************************************************************************
//* --------------------
//* S4CDCExactHit
//* --------------------
//* (Description)
//*	CDC Hit class.
//*	Mother class : TObject
//*     
//* (Update Record)
//*	2001/03/15  K.Hoshina	Original version.
//*************************************************************************

#include <iostream>
#include <iomanip>

#include "TObject.h"
#include "TVector3.h"
#include "TString.h"

#include "S4VExactHit.h"

#ifndef __CDCNWIRESPERCELL__
#define __CDCNWIRESPERCELL__  5
//#define __CDCNWIRESPERCELL__  7
#endif
#if 1
#include "TMath.h"
#endif

#define __CDCCELLIDORDER__    10000

class J4VCDCDriftRegionHit;

class S4CDCExactHit : public S4VExactHit {

public:
    
  S4CDCExactHit(){} 
  S4CDCExactHit(const J4VCDCDriftRegionHit *);
  S4CDCExactHit(      Int_t     evtNumber   , 
                      Int_t     layerNumber , 
                      Int_t     wireNumber  , 
                      Int_t     cellNumber  ,
                      Int_t     trackNumber ,
                      Int_t     mothertrackN,
                      Int_t     hitnumber   ,
                      Int_t     pcode       , 
                      Int_t     charge      ,
                      Double_t  cellDphi    , 
                      Double_t  cellSphi    , 
                      Double_t  wirePhi     , 
                      Double_t  wireR       ,
                      Double_t  driftlen    , 
                const TVector3 &pre         , 
                const TVector3 &post        , 
                const TVector3 &hit         , 
                const TVector3 &momentum    ,
                      Double_t  totE        , 
                      Double_t  edep        , 
                      Double_t  tof         ,
                const TVector3 &wireEndm    , 
                const TVector3 &wireEndp    ,
                      Double_t  tanstereo , 
                      Double_t  rwaist);  

  S4CDCExactHit(      Int_t     evtNumber   ,
                      Int_t     layerNumber ,
                      Int_t     wireNumber  ,
                      Int_t     cellNumber  ,
                      Int_t     trackNumber ,
                      Int_t     mothertrackN,
                      Int_t     hitnumber   ,
                      Int_t     pcode       ,
                      Int_t     charge      ,
                      Double_t  cellDphi    ,
                      Double_t  cellSphi    ,
                const TVector3 &pre         ,
                const TVector3 &post        ,
                const TVector3 &momentum    ,
                      Double_t  totE        ,
                      Double_t  edep        ,
                      Double_t  tof         ,
                const TVector3 &wireEndm    ,
                const TVector3 &wireEndp    ,
                      Double_t  tanstereo ,
                      Double_t  rwaist);

  inline       S4CDCExactHit (const S4CDCExactHit& right);    

  virtual ~S4CDCExactHit();
  
  virtual void Print(){};
  
  static void	SetSortBy(Char_t *opt = "WireLayer");
  Bool_t	IsSortable() const;
  Int_t         Compare(const TObject *obj) const;  
  
  
  //Get method -----------//
  
  inline virtual Int_t 	  GetLayerNumber() const { return fLayerNumber; }
  inline virtual Int_t 	  GetWireNumber () const { return fWireNumber ; }
  inline virtual Int_t 	  GetCellNumber () const { return fCellNumber ; }
  inline virtual Int_t 	  GetCellID     () const { return fCellID     ; }
  inline virtual Double_t GetCellDphi   () const { return fCellDphi   ; }
  inline virtual Double_t GetCellSphi   () const { return fCellSphi   ; }
  inline virtual Double_t GetDriftLen   () const { return fDriftLen   ; } 
  inline virtual Double_t GetWirePhi    () const { return fWirePhi    ; }
  inline virtual Double_t GetWireR      () const { return fWireR      ; }
  inline virtual Double_t GetTanStereo  () const { return fTanStereo  ; }
  inline virtual Double_t GetTan2Stereo () const { return fTanStereo * fTanStereo ; }
  inline virtual Double_t GetRwaist     () const { return fRwaist     ; }
  inline virtual Double_t GetRwaist2    () const { return fRwaist*fRwaist; }
  inline virtual Bool_t   GetFlag       () const { return fFlag       ; }
  inline virtual Double_t GetWireX      () const ;
  inline virtual Double_t GetWireY      () const ;
  inline virtual Int_t 	  GetDriftRegionNumber () const 
                                           { return fDriftRegionNumber; }  
  inline virtual const    TVector3 & GetWireBtm() const 
                                           { return fWireEndAtMinusZ  ; }
  inline virtual const    TVector3 & GetWireTop() const 
                                           { return fWireEndAtPlusZ   ; }
  inline virtual const    TVector3 & GetPrePosition() const 
                                           { return fPrePosition      ; }
  inline virtual const    TVector3 & GetPostPosition() const 
                                           { return fPostPosition     ; }
						
  
  //Set method -----------//
  
  inline virtual void SetLayerNumber(Int_t layerno) ;
  inline virtual void SetWireNumber (Int_t wireno)  ;
  inline virtual void SetCellNumber (Int_t cellno)  ;
  inline virtual void SetCellDphi(Double_t dphi)  { fCellDphi = dphi ; }
  inline virtual void SetCellSphi(Double_t sphi)  { fCellSphi = sphi ; }
  inline virtual void SetDriftLen(Double_t drlen) { fDriftLen = drlen; }
  inline virtual void SetWirePhi (Double_t wphi)  { fWirePhi  = wphi ; }
  inline virtual void SetWireR   (Double_t wr)    { fWireR    = wr   ; }
  inline virtual void SetWireBtm(const TVector3& wireEndm)
                      { fWireEndAtMinusZ = wireEndm; }
  inline virtual void SetWireTop (const TVector3& wireEndp)
                      { fWireEndAtPlusZ  = wireEndp; }
  inline virtual void SetFlag    (Bool_t f) { fFlag = f; }
  inline virtual void SetPrePosition (const TVector3 &x) { fPrePosition  = x; }
  inline virtual void SetPostPosition(const TVector3 &x) { fPostPosition = x; }

  inline Double_t     DistanceToLine(const TVector3 &p,
                                     const TVector3 &x0, 
                                     const TVector3 &d, 
                                           TVector3 &xx);

  
private:
 
  Int_t         fLayerNumber      ; //
  Int_t         fWireNumber       ; //
  Int_t         fCellNumber       ; //
  Int_t         fCellID           ; //
  Int_t         fDriftRegionNumber; //
  Double_t      fCellDphi         ; //
  Double_t      fCellSphi         ; //
  Double_t      fWirePhi          ; //
  Double_t      fWireR            ; //
  Double_t      fDriftLen         ; //
  TVector3      fWireEndAtMinusZ  ; //
  TVector3      fWireEndAtPlusZ   ; //
  Double_t      fTanStereo        ; //
  Double_t      fRwaist           ; //
  Bool_t        fFlag             ; //
  TVector3      fPrePosition      ; //
  TVector3      fPostPosition     ; //

  static Int_t	fCDCExactHitSortBy; //!
  
  	
 ClassDef(S4CDCExactHit, 1) // Datum of CDC Hits
};




//=====================================================================
//-----------------------------------
// inline function for S4CDCExactHit
//-----------------------------------

inline S4CDCExactHit::S4CDCExactHit(const S4CDCExactHit& right)
                     :S4VExactHit(right)
{

  fLayerNumber       = right.fLayerNumber       ;
  fWireNumber        = right.fWireNumber	;
  fCellNumber        = right.fCellNumber        ; 
  fCellID            = right.fCellID            ; 
  fCellDphi          = right.fCellDphi          ;  
  fCellSphi          = right.fCellSphi          ;
  fWirePhi           = right.fWirePhi           ;
  fWireR             = right.fWireR             ;
  fDriftLen          = right.fDriftLen          ;
  fTanStereo         = right.fTanStereo         ;  
  fRwaist            = right.fRwaist            ;  
  fWireEndAtMinusZ   = right.fWireEndAtMinusZ   ;
  fWireEndAtPlusZ    = right.fWireEndAtPlusZ    ;
  fDriftRegionNumber = right.fDriftRegionNumber ; 
            
}

//===========================================================
// Get method -----------------------------------------------

inline  Double_t S4CDCExactHit::GetWireX () const
{
	Double_t wirex = fWireR * cos(fWirePhi);
	return wirex;
}

inline  Double_t S4CDCExactHit::GetWireY () const
{
	Double_t wirey = fWireR * sin(fWirePhi);
	return wirey;
}

//===========================================================
// Set method -----------------------------------------------

inline  void S4CDCExactHit::SetLayerNumber (Int_t layerno) 
{
	fLayerNumber = layerno;
	fCellID	     
	   = fLayerNumber*__CDCCELLIDORDER__ + fCellNumber;
	fDriftRegionNumber 
	   = fLayerNumber*__CDCNWIRESPERCELL__ + fWireNumber;
}

inline  void S4CDCExactHit::SetWireNumber (Int_t wireno)
{
	fWireNumber = wireno;
	fDriftRegionNumber 
	   = fLayerNumber*__CDCNWIRESPERCELL__ + fWireNumber;
}

inline  void S4CDCExactHit::SetCellNumber (Int_t cellno) 
{
	fCellNumber = cellno;
	fCellID	     
	   = fLayerNumber*__CDCCELLIDORDER__ + fCellNumber;
}

//===========================================================
// DistanceToLine -------------------------------------------

Double_t S4CDCExactHit::DistanceToLine(const TVector3 &p,
                                       const TVector3 &x0,
                                       const TVector3 &d,
                                             TVector3 &xx)
{
   // DistanceToLine :
   // Calculate distance to line,
   // then return distance and global intersection points.
   //
   // p          - location of flying particle   
   // x0         - reference point of line
   // d          - direction vector of line
   // xx         - a foot of perpendicular line from p to the plane 
   // t          - distance from xx to p
   //
   // Equation
   //
   //    distance^2 = |(xx - p)|^2
   //    with
   //       xx = x0 + t*d
   //
   //   (d/dt)distance^2 = (d/dt)|((x0 + t*d) - p)|^2
   //                    = 2*t*|d|^2 + 2*d*(x0 - p)
   //                    = 0  // smallest distance
   //   then
   //      t = - d*(x0 - p) / |d|^2
   //

   Double_t t;
   TVector3 dir = d.Unit();
   t  = - dir * (x0 - p);      // |dir|^2 = 1.
   xx = x0 + t * dir;
   
   TVector3 dist = xx - p;
   return dist.Mag();
}




#endif
