#ifndef __S4BREAKPOINT__ 
#define __S4BREAKPOINT__ 
//*************************************************************************
//* --------------------
//* class S4BreakPoint
//* --------------------
//* (Description)
//*     Class to store break point information equivalent to J4BreakPoint.
//*    
//* (Update Record)
//*     2006/01/29  S.Yamamoto   Original version.
//*************************************************************************
//

#include "TLorentzVector.h"
#include "TObjArray.h"
#include "TAttLockable.h"
#include <map>

class J4BreakPoint;

// ************************************************************************
class S4BreakPoint : public TObjArray, public TAttLockable {
friend class S4HISTMCTruthBuf;
public:
  S4BreakPoint();
  S4BreakPoint(const J4BreakPoint* j4bpp);
  virtual ~S4BreakPoint();

  // Data Getters
  inline Int_t           GetTrackID()        const { return fTrackID;       }
  inline Int_t           GetParentID()       const { return fParentID;      }
  inline Double_t        GetCharge()         const { return fCharge;        }
  inline Int_t           GetParticleID()     const { return fParticleID;    }
  inline Double_t        GetGlobalTime()     const { return fGlobalTime;    }
  inline TVector3        GetPosition()       const { return fPosition;      }
  inline TLorentzVector  GetFourMomentum()   const { return fFourMomentum;  }
  inline Double_t        GetKineticEnergy()  const { return fKineticEnergy; }
  inline TObject        *GetPFOPtr()         const { return fPFOPtr;        }

  // Data Setter
   
  inline  void           SetPFOPtr(TObject *pfop) { fPFOPtr = pfop; }
  virtual void           LockAllDescendants();
          void           DebugPrint(const Char_t *opt = "");

  static S4BreakPoint  * GetBreakPointPtr(Int_t trackID);
  static void            Clear();
  static void            PrintAll(const Char_t *opt = "");
  static std::map<Int_t, S4BreakPoint*>
                       & GetBreakPointMap();

private:
  Int_t           fTrackID;         // track ID
  Int_t           fParentID;        // parent track ID
  Double_t        fCharge;          // charge
  Int_t           fParticleID;      // PDG code
  Double_t        fGlobalTime;      // Time since the collision
  TVector3        fPosition;        // positon of break point
  TLorentzVector  fFourMomentum;    // 4-momentum
  Double_t        fKineticEnergy;   // kinetic energy
  TObject        *fPFOPtr;          // pointer to pfo

  static std::map<Int_t, S4BreakPoint*> fgBreakPointMap; // TrackID to BreakPoint map

  ClassDef(S4BreakPoint, 3) // S4BreakPoint class
};

#endif
