//*************************************************************************
//* -------------------
//* S4HISTMCTruth Class
//* -------------------
//* (Description)
//*   	A class to store history information.
//* (Requires)
//*	S4VMCTruthBuf:
//* (Provides)
//*	S4HISTMCTruthBuf		: MCTruth data buffer
//* (Update Record)
//*	2006/02/03  K.Fujii	Original version.
//*************************************************************************

#include <iostream>
#include "S4HISTMCTruthBuf.h"
#include "S4VMCTruthBuf.h"
#include "S4BreakPoint.h"

using namespace std;

ClassImp(S4HISTMCTruthBuf)


//===========================================================
// Constructor -----------------------------------------------

S4HISTMCTruthBuf::S4HISTMCTruthBuf(const Char_t  *name,
                                   const Char_t  *title, 
                                   S4HISTMCTruth *module,
                                   Int_t          size) 
              : S4VMCTruthBuf(name, title, module, size),
                fInputBufPtr(0),
                fDone(kFALSE)
{
}

S4HISTMCTruthBuf::S4HISTMCTruthBuf(S4HISTMCTruth *module,
                                 const Char_t *name, 
                                 const Char_t *title,
                                 Int_t         size) 
              : S4VMCTruthBuf(name, title, module, size),
                fInputBufPtr(0),
                fDone(kFALSE)
{
   cerr << "constructor for S4HISTMCTruthBuf is called..." << endl;
}

//===========================================================
// Destructor -----------------------------------------------

S4HISTMCTruthBuf::~S4HISTMCTruthBuf()
{
}

//===========================================================
// ReadFile -----------------------------------------------

Bool_t S4HISTMCTruthBuf::ReadFile()
{
   return kTRUE;
}
 
//===========================================================
// Update  --------------------------------------------------

void S4HISTMCTruthBuf::Update()
{
   if (fDone) return;
   fDone = kTRUE;

   S4BreakPoint::Clear();

   TIter next(GetInputBufPtr());
   S4BreakPoint *bpp;
   while ((bpp = static_cast<S4BreakPoint *>(next()))) {
     S4BreakPoint::fgBreakPointMap.insert(
       std::pair<Int_t, S4BreakPoint*>(bpp->GetTrackID(), bpp));
       bpp->TObjArray::Clear();
   } 

   next.Reset();
   while ((bpp = static_cast<S4BreakPoint *>(next()))) {
     Int_t         j4ptkid = bpp->GetParentID();
     S4BreakPoint *pbpp    = S4BreakPoint::GetBreakPointPtr(j4ptkid);
     if (pbpp) pbpp->Add(bpp);
   } 
}
