#ifndef __S4ITEXACTHIT__
#define __S4ITEXACTHIT__
//*************************************************************************
//* --------------------
//* S4ITExactHit
//* --------------------
//* (Description)
//*	IT Hit class.
//*	Mother class : TObject
//*     
//* (Update Record)
//*	2001/03/15  K.Hoshina	Original version.
//*************************************************************************

#include "TObject.h"
#include "TVector3.h"

#include "S4VExactHit.h"

class J4ITLayerHit;

class S4ITExactHit : public S4VExactHit {

public:
    
  S4ITExactHit();
  S4ITExactHit(const J4ITLayerHit *hitp);
  S4ITExactHit(      Int_t     evtID,
                     Int_t     layerID,
                     Int_t     trackID,
                     Int_t     mothertrackID,
                     Int_t     hitID,
                     Int_t     pcode,
                     Int_t     charge,
               const TVector3 &pre,
               const TVector3 &post,
               const TVector3 &momentum,
                     Double_t  totE,
                     Double_t  edep,
                     Double_t  tof);

  virtual ~S4ITExactHit();
  
  virtual void DebugPrint(const Char_t *opt = "");
  
  //static void  SetSortBy(Char_t *opt = "TrackID");
  //Bool_t       IsSortable() const;
  //Int_t        Compare(const TObject *obj) const;  
  
  
  //Get method -----------//
  
  inline virtual Int_t 	  GetLayerID     () const { return fLayerID;      }
  inline virtual Int_t 	  GetSide        () const { return fSide;         }
  inline virtual Double_t GetLayerR      () const { return fLayerR;       } 
  inline virtual Double_t GetPhi         () const { return fPhi;          } 
  inline virtual TVector3 GetPrePosition () const { return fPrePosition;  }  
  inline virtual TVector3 GetPostPosition() const { return fPostPosition; }  

  //Set method -----------//
  inline virtual void SetLayerID     (Int_t    i)        { fLayerID      = i; }
  inline virtual void SetLayerR      (Double_t d)        { fLayerR       = d; }
  inline virtual void SetPhi         (Double_t d)        { fPhi          = d; }
  inline virtual void SetSide        (Int_t    i)        { fSide         = i; }
  inline virtual void SetPrePosition (const TVector3 &x) { fPrePosition  = x; }
  inline virtual void SetPostPosition(const TVector3 &x) { fPostPosition = x; }

private:
 
  Int_t         fLayerID;          //
  Double_t      fLayerR;           //
  Double_t      fPhi;              //
  Int_t         fSide;             //
  TVector3      fPrePosition;      //
  TVector3      fPostPosition;     //

  static Int_t	fITExactHitSortBy; //!
  
  	
  ClassDef(S4ITExactHit, 2) // Datum of IT Hits
};


#endif
