//*****************************************************************************
//* -------------------
//* S4VMCTruthBuf Class
//* -------------------
//*
//* (Description)
//*   This set of classes reads Monte Carlo ExactHit data 
//*   and put them into root objects.
//*   When a root file is opened in the read/write mode, the root
//*   objects are stored in it.  If the root file is read-only,
//*   the data are obtained from it.
//*
//* (Requires)
//*   TClonesArray:
//*
//* (Provides)
//*   S4VMCTruthBuf:            MCTruth ExactHit data buffer
//*
//* (Update Record)
//*   1999/06/02  K.Fujii       Original version.
//*
//*****************************************************************************

#include "JSFSteer.h"
#include "S4VMCTruthBuf.h"
#include "TRandom.h"
#include <iostream>
#include <sstream>
#if __GNUC__ >= 5 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3)
#include <cstdlib>
#endif

ClassImp(S4VMCTruthBuf)

extern JSFSteer *gJSF;

Int_t S4VMCTruthBuf::fgEventNo = -99999;

//_____________________________________________________________________________
// ----------------
// Class MCTruthBuf
// ----------------
//
//=============================================================================
//* Ctors and dtor    ---------------------------------------------------------
//
S4VMCTruthBuf::S4VMCTruthBuf(const Char_t     *name,
                             const Char_t     *title, 
			           S4VMCTruth *module,
                                   Int_t       bufsize) 
              : JSFEventBuf(name, title, (JSFModule*)module),
                fExactHits(bufsize)
{
}

S4VMCTruthBuf::S4VMCTruthBuf(      S4VMCTruth *module,
                             const Char_t     *name, 
			     const Char_t     *title,
                                   Int_t       bufsize) 
              : JSFEventBuf(name, title, (JSFModule*)module),
                fExactHits(bufsize)
{
}

S4VMCTruthBuf::~S4VMCTruthBuf()
{
   fExactHits.Delete();
}

//=============================================================================
//* Utility methods    --------------------------------------------------------
//
Int_t S4VMCTruthBuf::Seek(const Char_t *tag) 
{
   std::ifstream & in = *S4VMCTruth::GetInputStreamPtr();
   static Int_t kMaxBuf = 1024;
   Char_t  dummy[kMaxBuf];

   in.getline(dummy,kMaxBuf);

   Int_t    eventno = 0;
   while (fgEventNo < S4VMCTruth::fgEventToProcess) {
      while (strncmp(dummy,"*******_Start_of_event_", 23) && !in.eof()) {
         in.getline(dummy,kMaxBuf);
      }
      in.getline(dummy,kMaxBuf);
      std::istringstream ins(dummy);
      ins >> eventno;
      if (in.eof()) {
         std::cerr << "=================================="
                   << "=================================="  << std::endl
                   << " S4VMCTruthBuf::Seek : EOF reached!" << std::endl
                   << "=================================="
                   << "=================================="  << std::endl;
         gJSF->Terminate();
         gJSF->GetOutput()->Write();
         exit(0);
      }
      std::cerr << "=================================="
                << "==================================" << std::endl
                << " S4VMCTruthBuf::Seek : Event No " << eventno 
                << std::endl
                << "=================================="
                << "==================================" << std::endl;
      fgEventNo = eventno;
      if (eventno < S4VMCTruth::fgEventToProcess) in.getline(dummy,kMaxBuf);
   }

   in.getline(dummy,kMaxBuf);
  
   if (strncmp(dummy,"*****_EXPName:", 14)) {
      std::cerr << " S4VMCTruthBuf::Seek : "
                << " No detector tag. Stop. " 
                << std::endl;
      exit(1);
   }

   while (strncmp(dummy,tag, strlen(tag)-1) && !in.eof()) {
      if (! strncmp(dummy,"*******_End_of_event_" ,21) ) {
         std::cerr << " S4VMCTruthBuf::Seek : Got an EOE tag " 
                   << std::endl;
         return -1;
      }
      in.getline(dummy,kMaxBuf);
   }

   if (in.eof()) {
      std::cerr << " S4VMCTruthBuf::Seek : No output start tag." << std::endl
                << " " << tag << std::endl
                << " Stop. "  << std::endl;
      exit(1);
   }

   if ((in.peek() == '*')) {
      in.getline(dummy,kMaxBuf);
      return -1;
   }

#ifdef __DEBUG__
   std::cerr << " S4VMCTruthBuf::Seek : Got \"" << tag << "\"" << std::endl;
#endif

   return fgEventNo;
}
