///////////////////////////////////////////////////////////////////
//
//  MUD Hit class in ROOT
//
//$Id: S4MUDExactHit.cxx,v 1.2 2006/07/03 07:14:28 miyamoto Exp $
//
//////////////////////////////////////////////////////////////////

#include <iomanip>
#include <iostream>

#ifndef __HasJSFFromLCIO__
#include "J4MUDHit.hh"
#endif
#include "S4MUDExactHit.h"
using namespace std;

ClassImp(S4MUDExactHit)

// ------------------------------------------------------------------
S4MUDExactHit::S4MUDExactHit()
              : S4VExactHit(0,                 // hit ID
                            0,                 // track ID
                            0,                 // mother track ID
                            0,                 // PDG code
                            0,                 // charge
                            TVector3(0.,0.,0.),// position
                            0,                 // TOF
                            TVector3(0.,0.,0.),// momentum
                            0,                 // energy
                            0,                 // energy deposit
                            0),                // address ptr 
                fPreHitID(0),
                fBlockID(0),
                fLayerID(0),
                fIsBarrel(kTRUE),
                fIsFront(kTRUE),
                fPreHitPtr(0)
{
}

// ------------------------------------------------------------------
S4MUDExactHit::S4MUDExactHit(Int_t     hitID,       // hit ID
                             Int_t     pcode,       // PDG code
                      const TVector3  &pos,         // position [cm]
                             Double_t  tof,         // TOF      [nsec]
                             Double_t  edep,        // energy deposit [GeV]
                             Int_t     preHitID,    // pre hit ID
                             Int_t     blockID,     // block ID
                             Int_t     layerID,     // layer ID
                             Int_t     isBarrel,    // barrel or endcap
                             Int_t     isFront)     // front-endcap or endcap
              : S4VExactHit(hitID,             // hit ID
                            0,                 // track ID
                            0,                 // mother track ID
                            pcode,             // PDG code
                            0,                 // charge
                            pos,               // position
                            tof,               // TOF
                            TVector3(0.,0.,0.),// momentum
                            0,                 // energy
                            edep,              // energy deposit
                            0),                // address ptr 
                fPreHitID(preHitID),
                fBlockID(blockID),
                fLayerID(layerID),
                fIsBarrel(isBarrel),
                fIsFront(isFront),
                fPreHitPtr(0)
{
}

#ifndef __HasJSFFromLCIO__
// ------------------------------------------------------------------
S4MUDExactHit::S4MUDExactHit(J4MUDHit *ahit)
              : S4VExactHit(0,                          // hit ID
                            0,                          // track ID
                            0,                          // mother track ID
                            ahit->GetPDGEncoding(),     // PDG code
                            0,                          // charge
                            TVector3(ahit->GetPrePosition().x()/cm, // pre pos
                                     ahit->GetPrePosition().y()/cm,
                                     ahit->GetPrePosition().z()/cm),
                            ahit->GetTof()/nanosecond,  // TOF
                            TVector3(0.,0.,0.),         // momentum
                            0,                          // energy
                            ahit->GetEnergyDeposit()/GeV, // energy deposit
                            0),                         // address ptr 
                fPreHitID(ahit->GetPreHitID()),         // prehit ID
                fBlockID(ahit->GetBlockID()),           // block ID
                fLayerID(ahit->GetLayerID()),           // layer ID
                fIsBarrel(ahit->GetIsBarrel()),            // is Barrel?
                fIsFront(ahit->GetIsFront()),              // is Front?
                fPreHitPtr(0)
{
}
#endif

// ------------------------------------------------------------------
S4MUDExactHit::~S4MUDExactHit()
{
}

// ------------------------------------------------------------------
void S4MUDExactHit::Print(const Option_t *opt) const
{
  std::cout << std::setiosflags(std::ios::fixed);
  std::cout << "S4MUDExactHit ====================";
  //std::cout << " CellID=" << fCellID;
  std::cout << " BlockID=" << fBlockID; 
  std::cout << " LayerID=" << fLayerID; 
  std::cout << " PDGID=" << GetPDGCode();
  std::cout << " IsBarrel=" << fIsBarrel;
  std::cout << " IsFront=" << fIsFront;
  std::cout << " TOF(nsec)=" << GetTOF();
  std::cout << " EnergyDeposit(GeV)=" << GetEnergyDeposit();
  std::cout << std::endl;
  //std::cout << " CellPos(cm)=(" << fCellPosition.x();
  //std::cout << "," << fCellPosition.y();
  //std::cout << "," << fCellPosition.z();
  //std::cout << ") ";
  //std::cout << " CellXcm(GeV*cm)=(" << fCellXcm.x();
  //std::cout << "," << fCellXcm.y();
  //std::cout << "," << fCellXcm.z();
  //std::cout << ") ";
  //std::cout << std::endl;
}

//---------------------------------------------------------------
Int_t S4MUDExactHit::Compare(const TObject *hit) const
{
  return 0;
}

//---------------------------------------------------------------
void S4MUDExactHit::SetSortBy(Char_t *opt)
{
}

//---------------------------------------------------------------
Bool_t S4MUDExactHit::IsSortable() { return kTRUE; }
