//*************************************************************************
//* ----------------
//* S4MUDMCTruth Class
//* ----------------
//* (Description)
//* (Requires)
//*	S4VMCTruthBuf:
//* (Provides)
//*	S4MUDMCTruthBuf		: MCTruth data buffer
//* (Update Record)
//*	2004/11/24  S.Yamamoto	Original version.
//*************************************************************************

#include <sstream>
#include <iomanip>
#include <iostream>
#include "S4TPCMCTruthBuf.h"
#include "S4MUDMCTruthBuf.h"
#include "S4MUDMCTruthBuf.h"
#include "S4MUDExactHit.h"
#include "S4MUDPreHit.h"

#include "JSFSteer.h"

#define __MMTOCM__   0.100
#define __MEVTOGEV__ 0.001

using namespace std;

ClassImp(S4MUDMCTruthBuf)


//===========================================================
// Constructor -----------------------------------------------

S4MUDMCTruthBuf::S4MUDMCTruthBuf(const Char_t *name,
                                 const Char_t *title, 
                                 S4MUDMCTruth *module,
                                 Int_t         bufsize) 
               : S4VMCTruthBuf(name, title, module, bufsize),
                 fInputBufPtr(0),
                 fPreHitsPtr(0)
{
}

S4MUDMCTruthBuf::S4MUDMCTruthBuf(S4MUDMCTruth *module,
                                 const Char_t *name, 
                                 const Char_t *title,
                                 Int_t         bufsize) 
               : S4VMCTruthBuf(name, title, module, bufsize),
                 fInputBufPtr(0),
                 fPreHitsPtr(0)
{
   cerr << "constructor for S4MUDMCTruthBuf is called..." << endl;
}

//===========================================================
// Destructor -----------------------------------------------

S4MUDMCTruthBuf::~S4MUDMCTruthBuf()
{
}

//===========================================================
// ReadFile -------------------------------------------------
 
Bool_t S4MUDMCTruthBuf::ReadFile()
{
  cerr << " S4MUDMCTruthBuf::ReadFile() is called..." << endl;

  // Set Ownership Flag -------//
  GetExactHitsPtr()->SetOwner();
  if (!GetPreHitsPtr()) fPreHitsPtr = new TObjArray;
  GetPreHitsPtr()->SetOwner(); 

  ifstream & in = *S4VMCTruth::GetInputStreamPtr();
  // Seek MUD header -------//

  Int_t eventno = Seek("*****_EXPName:MUD_output_start_*****");
  if (eventno < 0)  return kTRUE;  // -ve means faiure to find one

  // read data -------//
  Char_t   dummy[1024];
  Int_t    preHitID, pcode, isBarrel;
  Int_t    blockID, layerID, isFront;
  Double_t cellx, celly, cellz, posx, posy, posz; 
  Double_t edep, tof;
  TVector3 pos;
  Int_t    postHitID, trackID, motherTrackID;
  Double_t etot, mx, my, mz, charge;
  TVector3 mom;
  	
  Int_t hitID = 0;    // very temporary!!!!

  while (!in.eof()) {
    in >> dummy;
    if (! strncmp(dummy,"*****_EXPName:MUD_output_end_*****" ,34) ) break;
    if (!strncmp(dummy,"prehit", 6)) { 
      in >> preHitID  >> pcode 
	 >> blockID  >> layerID  >> isBarrel >> isFront
	 >> edep  >> tof  
         >> posx  >> posy  >> posz ;
         //>> cellx >> celly >> cellz; 

      posx      *= __MEVTOGEV__ * __MMTOCM__;
      posy      *= __MEVTOGEV__ * __MMTOCM__;
      posz      *= __MEVTOGEV__ * __MMTOCM__;

      cellx     *= __MMTOCM__;
      celly     *= __MMTOCM__;
      cellz     *= __MMTOCM__;
      edep      *= __MEVTOGEV__;

      pos.SetX(posx/edep);
      pos.SetY(posy/edep);
      pos.SetZ(posz/edep);

   
      S4MUDExactHit *datum 
	= new S4MUDExactHit(hitID,                 // hit ID
			    pcode,                 // PDG code
			    pos,                   // position [cm]
                            tof,                   // TOF      [nsec]
                            edep,                  // energy deposit [GeV]
			    preHitID,              // pre hit ID
			    blockID,               // block ID
			    layerID,               // layer ID
                            isBarrel,              // is Barrel? 
                            isFront);              // is Front?

      GetExactHitsPtr()->Add(datum);	
      hitID++;  // very temporary!!!!

    } else {    // PretHit
      in >> postHitID >> preHitID 
         >> trackID   >> motherTrackID  
         >> etot 
	 >> mx        >> my      >> mz  
	 >> pcode     >> charge  >> tof
         >> posx      >> posy    >> posz ;
            
      pos.SetX(posx * __MMTOCM__);
      pos.SetY(posy * __MMTOCM__);
      pos.SetZ(posz * __MMTOCM__);

      mom.SetX(mx * __MEVTOGEV__);
      mom.SetY(my * __MEVTOGEV__);
      mom.SetZ(mz * __MEVTOGEV__);

      etot      *= __MEVTOGEV__;
 
      S4MUDPreHit *datum 
	= new S4MUDPreHit(postHitID,             // posthit ID
			  preHitID,              // prehit ID
			  trackID,               // track ID 
			  motherTrackID,         // mother track ID
			  pcode,                 // PDG code
			  charge,                // charge
			  pos,                   // Injection position [cm]
			  tof,                   // TOF      [nsec]
			  mom,                   // momentum [GeV] 
			  etot,                  // energy
			  edep,                  // energy deposit [GeV] 
			  0);                    // address ptr
	
      GetPreHitsPtr()->Add(datum);	
    }
  }

  return kTRUE;
}

//===========================================================
// UpdatePtrs -----------------------------------------------

void S4MUDMCTruthBuf::UpdatePtrs()
{
  TObjArray *ehitsp   = GetExactHitsPtr();  // exact hists collection
  TObjArray *prehitsp = GetPreHitsPtr();    // prehits collection

  TObjArray *poshitsp = static_cast<S4TPCMCTruthBuf *>(gJSF->FindModule("S4TPCMCTruth")->EventBuf())->GetPostHitsPtr(); // post hit collection

  if ( !prehitsp ) {
    //std::cerr << "MUDPreHits does not exist!" << std::endl;
    return;
  }

  TIter next(ehitsp);
  S4MUDExactHit *ehitp = 0;
  while ((ehitp = static_cast<S4MUDExactHit *>(next()))) {
    Int_t prehitid = ehitp->GetPreHitID();
    S4MUDPreHit  *prehitp = prehitid == -1 
                           ? 0
                           : static_cast<S4MUDPreHit *>(prehitsp->At(prehitid));

    ehitp->SetPreHitPtr(prehitp);

    if ( prehitp ) {
      Int_t poshitid = prehitp->GetPostHitID();
      S4TPCPostHit *poshitp = poshitid == -1 
                          ? 0
                          : static_cast<S4TPCPostHit *>(poshitsp->At(poshitid));
      prehitp->SetPostHitPtr(poshitp);
    }
  }
}
