#ifndef __S4TPCEXACTHIT__
#define __S4TPCEXACTHIT__
//*************************************************************************
//* --------------------
//* S4TPCExactHit
//* --------------------
//* (Description)
//*	TPC Hit class.
//*	Mother class : TObject
//*     
//* (Update Record)
//*	2001/03/15  K.Hoshina	Original version.
//*************************************************************************

#include "TVector3.h"

#include "S4VExactHit.h"

class J4TPCLayerHit;

class S4TPCExactHit : public S4VExactHit {

public:
  S4TPCExactHit();
  S4TPCExactHit(const J4TPCLayerHit *hitp);
  S4TPCExactHit(      Int_t     evtNumber,
                      Int_t     layerNumber,
                      Int_t     trackNumber,
                      Int_t     mothertrackN,
                      Int_t     hitnumber,
                      Int_t     pcode,
                      Int_t     charge,
                const TVector3 &pre,
                const TVector3 &post,
                const TVector3 &momentum,
                      Double_t  totE,
                      Double_t  edep,
                      Double_t  tof);

  virtual ~S4TPCExactHit();
  
  virtual void DebugPrint(const Char_t *opt = "");
  
  
  //Get method -----------//
  inline virtual Int_t 	  GetPadRowNumber() const { return fPadRowNumber; }
  inline virtual Double_t GetPadRowRadius() const { return fPadRowRadius; } 
  inline virtual Double_t GetPhi         () const { return fPhi         ; } 
  inline virtual Double_t GetZdrift      () const { return fZdrift      ; } 
  inline virtual Int_t    GetSide        () const { return fSide        ; }  
  inline virtual TVector3 GetPrePosition () const { return fPrePosition ; }  
  inline virtual TVector3 GetPostPosition() const { return fPostPosition; }  

  //Set method -----------//
  inline virtual void SetPadRowRadius(Double_t d)        { fPadRowRadius = d; }
  inline virtual void SetPhi         (Double_t d)        { fPhi          = d; }
  inline virtual void SetZdrift      (Double_t d)        { fZdrift       = d; }
  inline virtual void SetSide        (Int_t    n)        { fSide         = n; }
  inline virtual void SetPrePosition (const TVector3 &x) { fPrePosition  = x; }
  inline virtual void SetPostPosition(const TVector3 &x) { fPostPosition = x; }

private:
  Int_t         fPadRowNumber;      //
  Double_t      fPadRowRadius;      //
  Double_t      fPhi;               //
  Int_t         fSide;              //
  Double_t      fZdrift;            //
  TVector3      fPrePosition;       //
  TVector3      fPostPosition;      //
  
  	
  ClassDef(S4TPCExactHit, 2) // Datum of TPC Hits
};

#endif
