//*************************************************************************
//* --------------------
//* S4VTXExactHit
//* --------------------
//* (Description)
//*	VTX Hit class.
//*	Mother class : S4VExactHit
//*     
//* (Update Record)
//*	2004/12/16  T.Fujikawa	Original version.
//*************************************************************************

#ifndef __HasJSFFromLCIO__
#include "J4VTXPixelHit.hh"
#include "J4VTXLayerHit.hh"
#endif

#include <iostream>
#include "S4VTXExactHit.h"

using namespace std;

ClassImp(S4VTXExactHit)

//=====================================================================
//* Constructor -------------------------------------------------------
S4VTXExactHit::S4VTXExactHit():S4VExactHit(),
                               fPrePosition(0.,0.,0.), fPostPosition(0.,0.,0.),
                               fPhi(0.), fZ(0.), fR(0.),
			       fLayerID(0), fLadderID(0), 
			       fSensorID(0),fPixelThetaID(0), fPixelPhiID(0)
                                
{
}

#ifndef __HasJSFFromLCIO__
//=====================================================================
S4VTXExactHit::S4VTXExactHit(J4VTXPixelHit *ahit)
             : S4VExactHit(0,                            // hit ID
                           ahit->GetTrackID(),           // track ID
                           ahit->GetMotherTrackID(),     // mother track ID
                           ahit->GetPDGEncoding(),       // particle ID
                           (Int_t)ahit->GetCharge(),     // charge
                           TVector3(ahit->GetInPosition().x()/cm, 
                                           ahit->GetInPosition().y()/cm, 
                                           ahit->GetInPosition().z()/cm),
                           ahit->GetTof(),               // TOF
			   TVector3(ahit->GetPMomentum().x()/GeV, 
				    ahit->GetPMomentum().y()/GeV, 
				    ahit->GetPMomentum().z()/GeV),
                           ahit->GetTotalEnergy()/GeV,   // total energy
                           ahit->GetEnergyDeposit()/GeV, // energy deposit
                           0,                            // address ptr
                           0),                           // event ID 
               fPrePosition(TVector3(ahit->GetInPosition().x()/cm,
                                     ahit->GetInPosition().y()/cm,
                                     ahit->GetInPosition().z()/cm)),
               fPostPosition(TVector3(ahit->GetOutPosition().x()/cm,
                                      ahit->GetOutPosition().y()/cm,
                                      ahit->GetOutPosition().z()/cm)),
               fLayerID(ahit->GetLayerID()),          // Layer ID of Hit
	       fLadderID(ahit->GetLadderID()),        // Lader ID
	       fSensorID(ahit->GetSensorID()),        // Sensor ID
	       fPixelThetaID(ahit->GetPixThetaID()),  // Theta ID of pixel
	       fPixelPhiID(ahit->GetPixPhiID())       // PhiID of pixel
{
   fPhi = GetPosition().Phi();
   fZ   = GetPosition().Z();
   fR   = GetPosition().Pt();
}

//=====================================================================
S4VTXExactHit::S4VTXExactHit(J4VTXLayerHit *ahit)
             : S4VExactHit(0,                            // hit ID
                           ahit->GetTrackID(),           // track ID
                           ahit->GetMotherTrackID(),     // mother track ID
                           ahit->GetPDGEncoding(),       // particle ID
                           (Int_t)ahit->GetCharge(),     // charge
			   TVector3(ahit->GetInPosition().x()/cm, 
				         ahit->GetInPosition().y()/cm, 
				         ahit->GetInPosition().z()/cm),
                           ahit->GetTof(),               // TOF
			   TVector3(ahit->GetMomentum().x()/GeV, 
				    ahit->GetMomentum().y()/GeV, 
				    ahit->GetMomentum().z()/GeV),
                           ahit->GetTotalEnergy()/GeV,   // total energy
                           ahit->GetEnergyDeposit()/GeV, // energy deposit
                           0,                            // address ptr
                           0),                           // event ID 
               fPrePosition(TVector3(ahit->GetInPosition().x()/cm,
                                     ahit->GetInPosition().y()/cm,
                                     ahit->GetInPosition().z()/cm)),
               fPostPosition(TVector3(ahit->GetOutPosition().x()/cm,
                                      ahit->GetOutPosition().y()/cm,
                                      ahit->GetOutPosition().z()/cm)),
               fLayerID(ahit->GetLayerID()), // Layer ID of Hit
	       fLadderID(0),                 // Lader ID
	       fSensorID(0),                 // Sensor ID
	       fPixelThetaID(0),             // Theta ID of pixel
	       fPixelPhiID(0)                // PhiID of pixel
{
   fPhi = GetPosition().Phi();
   fZ   = GetPosition().Z();
   fR   = GetPosition().Pt();
}
#endif

//=====================================================================
S4VTXExactHit::S4VTXExactHit(const S4VExactHit &hit, 
			     const TVector3 &prepos, 
			     const TVector3 &postpos, 
			     Short_t layerID, Short_t ladderID,
			     Short_t sensorID, Short_t pixelThetaID,
			     Short_t pixelPhiID)
  : S4VExactHit(hit), fPrePosition(prepos), fPostPosition(postpos),
    fPhi(prepos.Phi()), fZ(prepos.Z()), fR(prepos.Pt()),
    fLayerID(layerID), fLadderID(layerID), fSensorID(sensorID),
    fPixelThetaID(pixelThetaID), fPixelPhiID(pixelPhiID)
{
}
			     
//=====================================================================
S4VTXExactHit::S4VTXExactHit(Int_t     evtID,
			     Int_t     layerNumber,
                             Int_t     trackID,
                             Int_t     mothertrackID,
                             Int_t     hitID, 
                             Int_t     pcode, 
                             Int_t     charge,
                             const TVector3 &prepos, 
                             const TVector3 &postpos, 
                             const TVector3 &momentum,
                             Double_t  totE, 
                             Double_t  edep, 
                             Double_t  tof,
			     Short_t layerID, Short_t ladderID,
			     Short_t sensorID, Short_t pixelThetaID,
			     Short_t pixelPhiID)
             : S4VExactHit(hitID, trackID,  mothertrackID, 
                           pcode, charge,   prepos,
                           tof,   momentum, totE, 
                           edep,  0,        evtID),
	       fPrePosition(prepos),
	       fPostPosition(postpos),
	       fPhi(prepos.Phi()), fZ(prepos.Z()), fR(prepos.Pt()),
	       fLayerID(layerID), fLadderID(layerID), fSensorID(sensorID),
	       fPixelThetaID(pixelThetaID), fPixelPhiID(pixelPhiID)
{
}

//=====================================================================
//* Destructor --------------------------------------------------------

S4VTXExactHit::~S4VTXExactHit()
{
}

//=====================================================================
//* Print -------------------------------------------------------------

void S4VTXExactHit::Print()
{

  cout << "------<S4VTXExactHit::Print>------"            << endl
       << "Track id:      = " << GetTrackID()             << endl
       << "PDG ID         = " << GetParticleID()          << endl
       << "Layer id       = " << GetLayerID()             << endl
       << "Ladder id      = " << GetLadderID()            << endl
       << "Sensor id      = " << GetSensorID()            << endl
       << "PixelTheta id  = " << GetPixelThetaID()        << endl
       << "PixelPhi id    = " << GetPixelPhiID()          << endl
       << "EDeposit(eV)   = " << GetEnergyDeposit()       << endl
       << "PMomentum(GeV) = " << GetMomentum().X() << ","
                              << GetMomentum().Y() << ","
                              << GetMomentum().Z() << ")" << endl
       << "Postion(cm)    = ("<< GetPosition().X() << ","
                              << GetPosition().Y() << ","
                              << GetPosition().Z() << ")" << endl
       << "Radius(cm)     = " << GetPosition().Pt()       << endl;
}

