#ifndef __S4BabyTRACKHIT__
#define __S4BabyTRACKHIT__

//*************************************************************************
//* ---------------------
//* S4BabyTrackHit Class
//* ---------------------
//* (Description)
//*	Class for a single track hit.
//* (Requires)
//*	TObjArray
//* (Provides)
//*	S4BabyTrackHit		: 
//* (Update Record)
//*	2002/08/28  R.Kuboshima
//*************************************************************************

class S4BabyVectorHit;

#include "U4CDCTrackHit.h"
#include "S4BabyHit.h"
#include "S4BabyExactHit.h"
#include "S4BabyVectorHit.h"

class S4BabyTrackHit : public U4CDCTrackHit {
friend class S4BabyVectorHit;

public:

  S4BabyTrackHit(S4BabyExactHit* exacthitp);
  virtual ~S4BabyTrackHit(); 
  
  Int_t			  Compare(const TObject * obj) const;
  virtual void 		  DebugPrint(const Char_t *opt = "Brief");
  inline  virtual void  	  Revive() ;
  inline  virtual Bool_t          IsKilled() const;
  
    
  inline  virtual S4BabyExactHit*  GetExactHitPtr();
  inline  virtual Int_t		  GetTrackID() const;
  inline  virtual Int_t		  GetLayerNumber() const;
  inline  virtual Int_t 	  GetCellNumber	();
  inline  virtual Int_t		  GetWireNumber () const;
  inline  virtual Int_t		  GetPDGCode    () const;
  
private:
  			    
  Bool_t          fKilled;		// flag to tell hits are killed or not.
  S4BabyExactHit  *fExactHitPtr;		// pointer to S4BabyExactHit.
  S4BabyVectorHit *fVectorHitPtr;	// pointer to S4BabyVectorHit.

  ClassDef(S4BabyTrackHit, 1) 	// Baby Hit Wire
};

//---------------------------------------
// inline function
//---------------------------------------

void S4BabyTrackHit::Revive() 
{
	fKilled = kFALSE;
}

Bool_t S4BabyTrackHit::IsKilled() const
{
	return fKilled;
}

S4BabyExactHit* S4BabyTrackHit::GetExactHitPtr()
{
	return fExactHitPtr;
}

Int_t S4BabyTrackHit::GetTrackID() const
{
	return fExactHitPtr->GetTrackID();
}

Int_t S4BabyTrackHit::GetLayerNumber() const
{
	return fExactHitPtr->GetLayerNumber();
}

Int_t S4BabyTrackHit::GetCellNumber()
{
	return fExactHitPtr->GetCellNumber();
}

Int_t S4BabyTrackHit::GetWireNumber() const
{
	return const_cast<S4BabyTrackHit *>(this)->GetHitPtr()->GetWireNumber();
}

Int_t S4BabyTrackHit::GetPDGCode() const
{
	return fExactHitPtr->GetPDGCode();
}


#endif


