//*************************************************************************
//* ---------------------
//* S4CALEMClusterMaker Class
//* ---------------------
//* (Description)
//*
//* (Requires)
//*
//* (Provides)
//*
//* (Update Record)
//*
//*************************************************************************
#include "S4CALSmallClusterUtils.h"
#include "S4CALClusterUtils.h"
#include "S4CALTrackMatcher.h"
#include "S4CALEMClusterMakerUtils.h"
#include "S4CALEMClusterMaker.h"
#include "S4TPCPostHit.h"
#include "S4CALPreHit.h"
#include "S4CALHit.h"
#include "S4CALExactHit.h"
#include "S4HYBTrackMakerBuf.h"
#include "S4CALAddress.h"

#include "JSFSteer.h"
#include "TNtuple.h"
#include <iostream>

//#define __LIKELIHOOD__

ClassImp(S4CALEMClusterMaker)


//=====================================================================
//* constructor -------------------------------------------------------
S4CALEMClusterMaker::S4CALEMClusterMaker()
  : fIsolationRin(6.), fIsolationRout(30.),
    fIsolationTubeHalfH(50.), fIsolationCut(4.),
    fChisquareCut(1.0), fFitterTubeRCut(5.), fFitterTubeHalfHCut(15.),
    fTrackDistanceCut(13.),
    fEHCompareTubeR(15.), fEHCompareTubeHalfH(60.), fEHCompareHERatioCut(0.1),
    fTubeRCut(5.), fTubeHalfHCut(15.), fTOFCut(14.),
    fMinLayerCut(12.)
{
}


//=====================================================================
//* destructor --------------------------------------------------------

S4CALEMClusterMaker::~S4CALEMClusterMaker()
{
}

//==========================================================================
//* MakeEMClusters ---------------------------------------------------------

void S4CALEMClusterMaker::MakeEMClusters(std::map<Int_t,S4CALRealCluster*> &clmap,
					 std::map<Int_t,S4CALSmallCluster*> &sclmap,
					 std::map<Int_t,S4HYBTrack*> &trkmap)
{

  MakePhotonClusters(clmap, sclmap, trkmap);

  // MakeSmallPhotonClusters(clmap, sclmap, trkmap); // low energy gamma

  //MakeElectronClusters(clmap, sclmap, trkmap);

}
#ifdef __LIKELIHOOD__
//==========================================================================
//* MakePhotonClusters -----------------------------------------------------

void S4CALEMClusterMaker::MakePhotonClusters(std::map<Int_t,S4CALRealCluster*> &clmap,
					     std::map<Int_t,S4CALSmallCluster*> &sclmap,
					     std::map<Int_t,S4HYBTrack*> &trkmap)
{
  /////////////////////////////////////////////////////////////
  static TNtuple* gamlike = new TNtuple("gamlike","likelihood test","e:en:ek:echd:egam:ee:eother:l");
  /////////////////////////////////////////////////////////////

  //----- cut values for all -----
  const Double_t vmin      = 18.;  // velocity
  const Double_t trkdismin = 0.;
  const Double_t lcut      = 0.7;   // likelihood cut
  //------------------------------

  //----- parameters for clustering -----
  const Double_t tubeR     = 5.7;
  //const Double_t tubeR     = 4.;  // for LDCSCL
  const Double_t tubeH     = 15.; // half H
  //-------------------------------------

  //------- parameters for likelihood calculation --------
  // parameters are trkdis, velocity, meanlayerID, edep/nhits....
  // (+ chi2/ndf, maxELayerID, depth(EMLongitudinalShape), angle, width...)
  static const Int_t nparams  = 5;
  static const Int_t mbgkinds = 1;
  //static const Int_t nbins    = 10; // default number of bins
  static const Int_t nbins    = 30; // default number of bins
  static TMatrixD sprob(nparams,1);
  static TMatrixD bgprob(nparams,mbgkinds);
  static Bool_t first = kTRUE;
  //------------------------------------------------------

  //-------- sample histograms ---------
  static std::map<Double_t,Double_t> gamtrkdis;
  static std::map<Double_t,Double_t> bgtrkdis;
  static std::map<Double_t,Double_t> gamv;
  static std::map<Double_t,Double_t> bgv;
  static std::map<Double_t,Double_t> gammeanlayer;
  static std::map<Double_t,Double_t> bgmeanlayer;
  static std::map<Double_t,Double_t> gamedepovern;
  static std::map<Double_t,Double_t> bgedepovern;
  static std::map<Double_t,Double_t> gamchi2;
  static std::map<Double_t,Double_t> bgchi2;
  static std::map<Double_t,Double_t> gammaxElayer;
  static std::map<Double_t,Double_t> bgmaxElayer;
  static std::map<Double_t,Double_t> gamlength;
  static std::map<Double_t,Double_t> bglength;
  //------------------------------------

  //------ parameters for trkdis ---------
  static Double_t strkdisprobmax  = 0.; // which maximize Ps/(Ps+Pbg) for trkdis
  static Double_t bgtrkdisprobmax = 0.;
  //--------------------------------------

  //---------- make sample histograms -----------
  if(first){
    first = kFALSE;
    TDirectory *last = gDirectory;
    last->cd();
    Int_t nzqqbar = 21;
    Char_t fzqqbar[21][512];
    TChain *tree = new TChain("gamtree");
    for(Int_t i=11;i<nzqqbar;i++){
      sprintf(fzqqbar[i],"/data4/jlc/tyosioka/jupiter/cvs.060614a/zqqbar/nt/uds91-jun06_2x2_2m-500-clx-%d.root",i);
      if (i!=3&&i!=17) {
	tree->Add(fzqqbar[i]);
      }
    }

    gamtrkdis    
      = S4CALClusterUtils::MakeSampleHistogram(tree,"trkdis","(egam)",nbins,90.);
    bgtrkdis     
      = S4CALClusterUtils::MakeSampleHistogram(tree,"trkdis","(edep-egam)",nbins,90.);
    gamv
      = S4CALClusterUtils::MakeSampleHistogram(tree,"v","(egam)",nbins,36.);
    bgv
      = S4CALClusterUtils::MakeSampleHistogram(tree,"v","(edep-egam)",nbins,36.);
    gammeanlayer 
      = S4CALClusterUtils::MakeSampleHistogram(tree,"meanlayerID","(egam)",nbins,42.);
    bgmeanlayer  
      = S4CALClusterUtils::MakeSampleHistogram(tree,"meanlayerID","(edep-egam)",nbins,42.);
    gamedepovern 
      = S4CALClusterUtils::MakeSampleHistogram(tree,"edep/nhits","(egam)",nbins,0.0045);
    bgedepovern  
      = S4CALClusterUtils::MakeSampleHistogram(tree,"edep/nhits","(edep-egam)",nbins,0.0045);
    gamchi2 
      = S4CALClusterUtils::MakeSampleHistogram(tree,"chi2","(egam)",nbins,3.);
    bgchi2  
      = S4CALClusterUtils::MakeSampleHistogram(tree,"chi2","(edep-egam)",nbins,3.);
    gammaxElayer 
      = S4CALClusterUtils::MakeSampleHistogram(tree,"maxELayerID","(egam)",nbins,60.);
    bgmaxElayer  
      = S4CALClusterUtils::MakeSampleHistogram(tree,"maxELayerID","(edep-egam)",nbins,60.);
    gamlength
      = S4CALClusterUtils::MakeSampleHistogram(tree,"maxlayerID-minlayerID","(egam)",nbins,60.);
    bglength  
      = S4CALClusterUtils::MakeSampleHistogram(tree,"maxlayerID-minlayerID","(edep-egam)",nbins,3.);

  }
  //---------------------------------------------

  std::map<Int_t, S4CALSmallCluster*>::iterator itr;
  std::multimap<Double_t, S4CALSmallCluster*>::reverse_iterator titr;
  std::vector<Int_t> idvec; // scl ID list which is not regarded as photon scl

  //--- sort sclmap --------
  std::multimap<Double_t,S4CALSmallCluster*> tmpmap;
  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){
    if(itr->second->IsLocked() || itr->second->IsMIP()) continue;
    tmpmap.insert(std::make_pair(itr->second->GetEnergy(),itr->second));
  }
  //------------------------

  //--------------- Make and Add clusters ------------------

  for(titr = tmpmap.rbegin(); titr != tmpmap.rend(); titr++){

    Int_t mID = titr->second->GetClusterID();

    if(titr->second->IsLocked())     continue;
    //    if(titr->second->GetNHits() < 5) continue;
    if(!(titr->second->IsEM()) ){
      titr->second->Lock();
      idvec.push_back(mID);
      continue;
    }

    TVector3 mpos    = titr->second->GetPosition();
    TVector3 mthrust = titr->second->GetThrustVector();

#if 0
    if(mthrust.Mag() < 1.e-6){
      titr->second->Lock();
      idvec.push_back(mID);
      continue;
    }
#endif

    //------------ make a new real cluster ----------------
    S4CALRealCluster *cl = new S4CALRealCluster();
    std::vector<S4CALSmallCluster*> sclvec;
    for(itr = sclmap.begin(); itr != sclmap.end(); itr++){
      if(itr->second->IsLocked()) continue;
      //if(S4CALClusterUtils::IsWithinTube(itr->second->GetPosition(), mpos, mthrust, 7., 15.)){
      if(S4CALClusterUtils::IsWithinTube(itr->second->GetPosition(), mpos, mpos, tubeR, tubeH)){
	cl -> Add(itr->second);
	sclvec.push_back(itr->second); // don't lock scls at here
      }
    }
    cl->Update();
    //-----------------------------------------------------

#if 0
    if(cl->GetNHits() < 11){
      titr->second->Lock(); // don't lock mother before checking chi2 etc!
      idvec.push_back(mID);
      delete cl;
      continue;
    }
#endif

    //------------ calculte probabilities --------------
    // The parameters are [0]:trkdis, [1]:velocity, [2]:meanlayerID,
    // [3]:edep/nhits.

    //Double_t mdist = (S4CALSmallClusterUtils::NearestTrackIs(trkmap,mpos)).second;
    Double_t dist        = (S4CALSmallClusterUtils::NearestTrackIs(trkmap,cl->GetPosition())).second;
    Double_t velocity    = (cl->GetPosition()).Mag()/cl->GetTime();
    Double_t meanlayerID = cl->GetMeanLayerID();
    Double_t edep        = cl->GetEnergyDeposit();
    Int_t nhits          = cl->GetNHits();
    S4CALEMClusterMakerUtils *emutil = new S4CALEMClusterMakerUtils();
    (titr->second->GetPFOMap()).erase(kgEMFitter);
    std::vector<Int_t> vec = emutil->EMFitter(sclmap,
                                              titr->second,
                                              tubeR,
                                              tubeH);
    Double_t chi2 = -100.;
    if( vec.size() > 0 )
      chi2 = (titr->second->GetPFOMap().find(kgEMFitter)->second)[3];
    Int_t maxELayerID = cl->GetMaxELayerID();
    Int_t length = cl->GetMaxLayerID() - cl->GetMinLayerID();

    //----- loose cut for cpu time saving -----
    //    if(velocity < vmin || dist < trkdismin || edep < 2.e-3){
    //      titr->second->Lock(); // don't lock mother before checking chi2 etc!
    //      idvec.push_back(mID);
    //      delete cl;
    //      continue;
    //    }
    //-----------------------------------------

    sprob(0,0)  = S4CALClusterUtils::CalcProbability(dist,gamtrkdis);
    sprob(1,0)  = S4CALClusterUtils::CalcProbability(velocity,gamv);
    sprob(2,0)  = S4CALClusterUtils::CalcProbability(meanlayerID,gammeanlayer);
    sprob(3,0)  = S4CALClusterUtils::CalcProbability(edep/nhits,gamedepovern);
    sprob(4,0)  = S4CALClusterUtils::CalcProbability(chi2,gamchi2);
    //    sprob(5,0)  = S4CALClusterUtils::CalcProbability(maxELayerID,gammaxElayer);
    //    sprob(6,0)  = S4CALClusterUtils::CalcProbability(length,gamlength);

    bgprob(0,0)  = S4CALClusterUtils::CalcProbability(dist,bgtrkdis);
    bgprob(1,0)  = S4CALClusterUtils::CalcProbability(velocity,bgv);
    bgprob(2,0)  = S4CALClusterUtils::CalcProbability(meanlayerID,bgmeanlayer);
    bgprob(3,0)  = S4CALClusterUtils::CalcProbability(edep/nhits,bgedepovern);
    bgprob(4,0)  = S4CALClusterUtils::CalcProbability(chi2,bgchi2);
    //    bgprob(5,0)  = S4CALClusterUtils::CalcProbability(maxELayerID,bgmaxElayer);
    //    bgprob(6,0)  = S4CALClusterUtils::CalcProbability(length,bglength);

    //    if(dist > 100.){
    //      sprob(0,0)  = strkdisprobmax;
    //      bgprob(0,0) = bgtrkdisprobmax;
    //    }
    //--------------------------------------------------

    Double_t likelihood = S4CALClusterUtils::CalcLikelihood(nparams,mbgkinds,sprob,bgprob);

    ///////////////////////// Tree /////////////////////////////
    Int_t    tpid = 0;
    Double_t tedep   = cl->GetEnergyDeposit();
    Double_t ten     = 0.;
    Double_t tek     = 0.;
    Double_t techd   = 0.;
    Double_t tegam   = 0.;
    Double_t tee     = 0.;
    Double_t teother = 0.;
    std::map<Int_t, Double_t>::iterator piditr;
    for(piditr = (cl->GetPIDMap()).begin(); piditr != (cl->GetPIDMap()).end(); piditr++){
      
      tpid = piditr->first;
      if(TMath::Abs(tpid) == 211)       techd += piditr->second;
      else if(TMath::Abs(tpid) == 22)   tegam += piditr->second;
      else if(TMath::Abs(tpid) == 11)   tee   += piditr->second;
      else if(TMath::Abs(tpid) == 130)  tek   += piditr->second;
      else if(TMath::Abs(tpid) == 2212) techd += piditr->second;
      else if(TMath::Abs(tpid) == 2112) ten   += piditr->second;
      else if(TMath::Abs(tpid) == 321)  techd += piditr->second;
      else  teother += piditr->second;
    }

    gamlike->Fill(tedep,ten,tek,techd,tegam,tee,teother,likelihood);

    //    std::cout << "likelihood = " << likelihood
    //	      << ", gam fraction = " << (tegam)/tedep << std::endl;
    ////////////////////////////////////////////////////////////////

#if 1
    if(likelihood < lcut){
      titr->second->Lock(); // don't lock mother before checking chi2 etc!
      idvec.push_back(mID);
      delete cl;
      continue;
    }
    //---------- set parameters of real cluster -----------
    for(Int_t k = 0; k < (Int_t) sclvec.size(); k++)  sclvec[k]->Lock();

    cl -> SetClusterType(kgEmNeutral_cl);
    cl -> SetParticleType(kgPhoton_p);
    cl -> Update();
    //-----------------------------------------------------

    clmap.insert(std::pair<Int_t, S4CALRealCluster*>(cl->GetClusterID(), cl));

#else
    //* for likelihood test
    titr->second->Lock(); // don't lock mother before checking chi2 etc!
    idvec.push_back(mID);
    delete cl;
    continue;
#endif

  }

  //--------------------------------------------------------

  for(Int_t i = 0; i < (Int_t) idvec.size(); i++){
    sclmap.find(idvec[i])->second->Unlock();
  }
  
}
#else
//==========================================================================
//* MakePhotonClusters -----------------------------------------------------

void S4CALEMClusterMaker::MakePhotonClusters(std::map<Int_t,S4CALRealCluster*> &clmap,
					     std::map<Int_t,S4CALSmallCluster*> &sclmap,
					     std::map<Int_t,S4HYBTrack*> &trkmap)
{

#define __GAMFITTER__
//#define __ANGLE__

  //--- cut values ------
  Double_t tof       = 16.;
  //Double_t tubeR     = 7.;  // default
  Double_t tubeR     = 5.7;
  //Double_t tubeR     = 4.;   // for LDCSCL
  Double_t tubeH     = 15.; // half H
  Double_t trkdis    = 7.;  // trk distance for small cluster
  Double_t depthcut  = 18.;
  //+ nhits, chi2, maxelayerID, velocity
  Double_t trkdisr   = 7.;  // trk distance for real cluster
  Double_t meanlayer = 14.;
  //---------------------



  S4CALEMClusterMakerUtils *emutil = new S4CALEMClusterMakerUtils();
  S4CALClusterUtils *ut = new S4CALClusterUtils();

  std::map<Int_t, S4CALSmallCluster*>::iterator itr;
  std::multimap<Double_t, S4CALSmallCluster*>::reverse_iterator titr;
  std::vector<Int_t> idvec;

  //--- sort sclmap --------
  std::multimap<Double_t,S4CALSmallCluster*> tmpmap;
  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){
    if(itr->second->IsLocked() || itr->second->GetTOF() > tof || itr->second->IsMIP()) continue;
    tmpmap.insert(std::make_pair(itr->second->GetEnergy(),itr->second));
  }
  //------------------------


  //--------------- Make and Add clusters ------------------

  for(titr = tmpmap.rbegin(); titr != tmpmap.rend(); titr++){

    Int_t mID = titr->second->GetClusterID();

    if(titr->second->IsLocked())     continue;
    if(titr->second->GetNHits() < 5) continue;
    if(!(titr->second->IsEM()) ){
      titr->second->Lock();
      idvec.push_back(mID);
      continue;
    }


    TVector3 mpos    = titr->second->GetPosition();
    TVector3 mthrust = titr->second->GetThrustVector();

    if(mthrust.Mag() < 1.e-6){
      titr->second->Lock();
      idvec.push_back(mID);
      continue;
    }


    //------------ make a new real cluster ----------------
    S4CALRealCluster *cl = new S4CALRealCluster();
    std::vector<S4CALSmallCluster*> sclvec;
    for(itr = sclmap.begin(); itr != sclmap.end(); itr++){
      if(itr->second->IsLocked()) continue;
      //if( ut->IsWithinTube(itr->second->GetPosition(), mpos, mthrust, 7., 15.) ){
      if( ut->IsWithinTube(itr->second->GetPosition(), mpos, mpos, tubeR, tubeH) ){
	cl -> Add(itr->second);
	sclvec.push_back(itr->second);
      }
    }
    cl->Update();
    //-----------------------------------------------------

#if 1
    if(cl->GetNHits() < 11){
      titr->second->Lock(); // don't lock mother before checking chi2 etc!
      idvec.push_back(mID);
      delete cl;
      continue;
    }
#endif

    //--- cut 1: The nearest track distance cut -----
    //Double_t dist = ( S4CALSmallClusterUtils::NearestTrackIs(trkmap,mpos) ).second;
    Double_t dist = ( S4CALSmallClusterUtils::NearestTrackIs(trkmap,cl->GetPosition()) ).second;
    //if(dist < fTrackDistanceCut){
    if(dist < trkdis){
      titr->second->Lock(); // don't lock mother before checking chi2 etc!
      idvec.push_back(mID);
      delete cl;
      continue;
    }
    //-----------------------------------------------


    //--------- cut 2. EMLongitudinalShape ----------
    std::vector<Int_t> longitudinal; 
    longitudinal.clear();
    Double_t depth = 1.e+5;
    //longitudinal = emutil->EMLongitudinalShape(sclmap, titr->second, 5.);
    longitudinal = emutil->EMLongitudinalShape(sclmap, titr->second, tubeR);
    if(longitudinal.size() > 0){

      depth = (titr->second->GetPFOMap().find(kgEMLongitudinal)->second)[1];
      if(depth > depthcut){
	titr->second->Lock();
	idvec.push_back(mID);
	delete cl;
	continue;
      }

    }else{

      titr->second->Lock();
      idvec.push_back(mID);
      delete cl;
      continue;

    }
    //-----------------------------------------------


#ifdef __GAMFITTER__
    //------- cut 3: EMFitter Chi-square cut --------
    (titr->second->GetPFOMap()).erase(kgEMFitter);
    //std::vector<Int_t> vec = emutil->EMFitter(sclmap,titr->second,fFitterTubeRCut,fFitterTubeHalfHCut);
    std::vector<Int_t> vec = emutil->EMFitter(sclmap,
					      titr->second,
					      tubeR,
					      tubeH);
    if( vec.size() > 0 ){

      Double_t chi2 = (titr->second->GetPFOMap().find(kgEMFitter)->second)[3];
      if( chi2 > 2.7){
	// Lock only mother small cluster
	titr->second->Lock();
	idvec.push_back(mID);
	delete cl;
	continue;
      }

    }else{
      titr->second->Lock();
      idvec.push_back(mID);
      delete cl;
      continue;
    }
    //-----------------------------------------------
#endif

    //---------- set parameters of real cluster -----------
    for(Int_t k = 0; k < (Int_t) sclvec.size(); k++)  sclvec[k]->Lock();

    cl -> SetClusterType(kgEmNeutral_cl);
    cl -> SetParticleType(kgPhoton_p);
    cl -> Update();
    //-----------------------------------------------------


    //---------- cut 4: Max E-layer cut -------------
    Double_t rclx = cl->GetPosition().X();
    Double_t rcly = cl->GetPosition().Y();
    Double_t rclz = cl->GetPosition().Z();
    Double_t rclt = cl->GetTime();

    Double_t vel     = TMath::Sqrt(rclx*rclx+rcly*rcly+rclz*rclz)/rclt;
    Double_t clnhits = (Double_t) cl->GetNHits();
    if( (cl->GetMaxELayerID() >= 30) || (cl->GetMaxELayerID() < 1)
	//|| (vel < 25.5) || (vel < -13./TMath::Sqrt(clnhits) + 30.) ){
       || (vel < 25.5) || (vel < -40./clnhits + 29.) ){
      for(Int_t l = 0; l < (Int_t) sclvec.size(); l++)   sclvec[l]->Unlock();
      titr->second->Lock();
      idvec.push_back(mID);
      delete cl;
      continue;
    }
    //-----------------------------------------------

    //------------- cut ?: angle cut ----------------
#ifdef __ANGLE__
    Double_t angle  = -1.;
    Double_t length = -1.;
    TVector3 clthv = cl->GetThrustVector();
    cl -> SetThrustVector(clthv);
    length = emutil->CalcClusterLength(cl, 0.8);
    if(clthv.Mag() > 1.e-10){
      angle = mpos.Angle(clthv);
      if(TMath::Abs(angle) > 10.0 && length > 40.){
	for(Int_t l = 0; l < (Int_t) sclvec.size(); l++)   sclvec[l]->Unlock();
	titr->second->Lock();
	idvec.push_back(mID);
	delete cl;
	continue;
      }
    }
#endif
    //-----------------------------------------------

    //////////////////////////////////////////////////////////////////
    //* def.
#if 0
    //------ cut ? trk dis, meanlayerID cut ---------
    Double_t distr = (S4CALSmallClusterUtils::NearestTrackIs(trkmap,cl->GetPosition())).second;
    if(distr < trkdisr || (cl->GetMeanLayerID() > meanlayer && cl->GetEnergyDeposit() < 0.1)){
      for(Int_t l = 0; l < (Int_t) sclvec.size(); l++)   sclvec[l]->Unlock();
      titr->second->Lock(); // lock only mother small cluster
      idvec.push_back(mID);
      delete cl;
      continue;
    }
    //-----------------------------------------------
#elif 0
    // meanlayerID 2
    //------ cut ? trk dis, meanlayerID cut ---------
    Double_t distr = (S4CALSmallClusterUtils::NearestTrackIs(trkmap,cl->GetPosition())).second;
    if(distr < 8.5 || (cl->GetMeanLayerID() > 15. && cl->GetEnergyDeposit() < 0.1)){
      for(Int_t l = 0; l < (Int_t) sclvec.size(); l++)   sclvec[l]->Unlock();
      titr->second->Lock(); // lock only mother small cluster
      idvec.push_back(mID);
      delete cl;
      continue;
    }
    //-----------------------------------------------
#elif 0
    // meanlayerID 3
    //------ cut ? trk dis, meanlayerID cut ---------
    Double_t distr = (S4CALSmallClusterUtils::NearestTrackIs(trkmap,cl->GetPosition())).second;
    if((cl->GetMeanLayerID() > 17. && cl->GetEnergyDeposit() < 0.1)){
      for(Int_t l = 0; l < (Int_t) sclvec.size(); l++)   sclvec[l]->Unlock();
      titr->second->Lock(); // lock only mother small cluster
      idvec.push_back(mID);
      delete cl;
      continue;
    }
    //-----------------------------------------------
#elif 1
    // meanlayerID 4
    //------ cut ? trk dis, meanlayerID cut ---------
    if(cl->GetMeanLayerID() > 17.5 && cl->GetEnergyDeposit() < 0.3){
      for(Int_t l = 0; l < (Int_t) sclvec.size(); l++)   sclvec[l]->Unlock();
      titr->second->Lock(); // lock only mother small cluster
      idvec.push_back(mID);
      delete cl;
      continue;
    }
    //-----------------------------------------------
#endif
    /////////////////////////////////////////////////////////////////

    /////////////////////////////////////////////////////////////////
    //* def.
#if 0
    //------------ cut ? edep:nhits cut -------------
    Double_t eovern = cl->GetEnergyDeposit()/cl->GetNHits();
    if(eovern > (0.001/20)*(cl->GetNHits()) + 0.0016
       || eovern < (0.0003/20)*(cl->GetNHits())){
      for(Int_t l = 0; l < (Int_t) sclvec.size(); l++)   sclvec[l]->Unlock();
      titr->second->Lock();
      idvec.push_back(mID);
      delete cl;
      continue;
    }
    //-----------------------------------------------
#elif 1
    //* edep/nhits 2
    //------------ cut ? edep:nhits cut -------------
    Double_t eovern = cl->GetEnergyDeposit()/cl->GetNHits();
    if(eovern > (0.008/20)*(cl->GetNHits()) + 0.0022
       || eovern < (0.00028/20)*(cl->GetNHits()) - 0.0002){
      for(Int_t l = 0; l < (Int_t) sclvec.size(); l++)   sclvec[l]->Unlock();
      titr->second->Lock();
      idvec.push_back(mID);
      delete cl;
      continue;
    }
    //-----------------------------------------------
#endif
    /////////////////////////////////////////////////////////////////

    clmap.insert(std::pair<Int_t, S4CALRealCluster*>(cl->GetClusterID(), cl));

  }

  //--------------------------------------------------------

  for(Int_t i = 0; i < (Int_t) idvec.size(); i++){
    sclmap.find(idvec[i])->second->Unlock();
  }
  
  delete emutil;
  delete ut;
  
}
#endif
//==========================================================================
//* MakeSmallPhotonClusters ------------------------------------------------

void S4CALEMClusterMaker::MakeSmallPhotonClusters(std::map<Int_t,S4CALRealCluster*> &clmap,
						  std::map<Int_t,S4CALSmallCluster*> &sclmap,
						  std::map<Int_t,S4HYBTrack*> &trkmap)
{

#define __SMALLTUBE__

  S4CALEMClusterMakerUtils *emutil = new S4CALEMClusterMakerUtils();
  S4CALClusterUtils *ut = new S4CALClusterUtils();

  std::map<Int_t, S4CALSmallCluster*>::iterator itr;
  std::multimap<Double_t, S4CALSmallCluster*>::reverse_iterator ritr;
  std::vector<Int_t> idvec;


  //------ sort sclmap --------
  std::multimap<Double_t,S4CALSmallCluster*> tmpmap;
  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){
    if(itr->second->IsLocked() || itr->second->GetTOF() > 15. || itr->second->IsMIP()) continue;
    tmpmap.insert(std::make_pair(itr->second->GetEnergy(),itr->second));
  }
  //-------------------------


  //--------------- Make and Add clusters ------------------

  for(ritr = tmpmap.rbegin(); ritr != tmpmap.rend(); ritr++){

    if(ritr->second->IsLocked() || !(ritr->second->IsEM()) ) continue;

    Int_t    mID     = ritr->second->GetClusterID();
    TVector3 mpos    = ritr->second->GetPosition();
    TVector3 mthrust = ritr->second->GetThrustVector();

    S4CALRealCluster *cl = new S4CALRealCluster();

    //------------ make a new real cluster ----------------
    std::vector<S4CALSmallCluster*> sclvec;    
    //--- add small clusters within small tube ----
    for(itr = sclmap.begin(); itr != sclmap.end(); itr++){

      if(itr->second->IsLocked()) continue;

#ifdef __SMALLTUBE__
      // small tube
      //if( ut->IsWithinTube(itr->second->GetPosition(),mpos,mpos,7.,10.) ){
      if( ut->IsWithinTube(itr->second->GetPosition(),mpos,mpos,5.7,10.) ){
	cl -> Add(itr->second);
	sclvec.push_back(itr->second);
      }
#else
      // dX dT
      Double_t mt   = ritr->second->GetTOF();
      TVector3 cpos = itr->second->GetPosition();
      Double_t ct   = itr->second->GetTOF();
      if( (mpos-cpos).Mag() < 10. && TMath::Abs(mt-ct) < 3.){
	cl->Add(itr->second);
	sclvec.push_back(itr->second);
      }
#endif
    }

    cl -> SetClusterType(kgEmNeutral_cl);
    cl -> SetParticleType(kgPhoton_p);
    cl -> Update();
    //---------------------------------------------

    //---- Lock only mother ----
    ritr->second->Lock();
    //--------------------------

    //---- cut 1 ----
    Double_t trkdis = (S4CALSmallClusterUtils::NearestTrackIs(trkmap,cl->GetPosition())).second;
    if(trkdis < 30.){
      idvec.push_back(mID);
      delete cl;
      continue;
    }
    //---------------

    Int_t nhits = cl->GetNHits();

    //---- cut 2 ----
    if(nhits > 15){
      idvec.push_back(mID);
      delete cl;
      continue;
    }
    //---------------

    ///////////////////////////////////////////////////////////////////
    //* def.
#if 0
    //*edep/nhits
    //---- cut 3 ----
    Double_t eovern = cl->GetEnergyDeposit()/nhits;
    if((eovern > (0.001/20)*nhits + 0.0016) || (eovern < (0.0003/20)*nhits)){
      idvec.push_back(mID);
      delete cl;
      continue;
    }
    //---------------
#elif 1
    //* edep/nhits 2
    Double_t eovern = cl->GetEnergyDeposit()/nhits;
    if(eovern > (0.008/20)*(cl->GetNHits()) + 0.0022
       || eovern < (0.00028/20)*(cl->GetNHits()) - 0.0002){
      idvec.push_back(mID);
      delete cl;
      continue;
    }
#else
    //* default
    //---- cut 3 ----
    Double_t edep = cl->GetEnergyDeposit();
    //if(edep/nhits > 0.04){
    if(edep/nhits > 0.0025){
      idvec.push_back(mID);
      delete cl;
      continue;
    }
    //---------------
#endif
    ///////////////////////////////////////////////////////////////////

    //---- cut 4 ----
    Double_t x = (cl->GetPosition()).X();
    Double_t y = (cl->GetPosition()).Y();
    Double_t z = (cl->GetPosition()).Z();
    if(!(TMath::Sqrt(x*x + y*y) < 225. && TMath::Abs(z) < 285.)){
      idvec.push_back(mID);
      delete cl;
      continue;
    }
    //---------------

    //---- cut 5 ----
    Double_t vel     = TMath::Sqrt(x*x+y*y+z*z)/(cl->GetTime());
    Double_t clnhits = (Double_t) cl->GetNHits();
    //if( (vel < 25.5) || (vel < -13./TMath::Sqrt(clnhits) + 30.) ){
    if( (vel < 25.5) || (vel < -40./clnhits + 29.) ){
      idvec.push_back(mID);
      delete cl;
      continue;
    }
    //---------------

    //* def.
#if 0
    //* meanlayerID 1
    //---- cut 6 ----
    if(cl->GetMeanLayerID() > 15.){
      idvec.push_back(mID);
      delete cl;
      continue;
    }
    //---------------
#elif 0
    //* meanlayerID 2
    //---- cut 6 ----
    if(cl->GetMeanLayerID() > 15.){
      idvec.push_back(mID);
      delete cl;
      continue;
    }
    //---------------
#elif 0
    //* meanlayerID 3
    //---- cut 6 ----
    if(cl->GetMeanLayerID() > 17.){
      idvec.push_back(mID);
      delete cl;
      continue;
    }
    //---------------
#elif 1
    //* meanlayerID 4
    //---- cut 6 ----
    if(cl->GetMeanLayerID() > 17.5 && cl->GetEnergyDeposit() < 0.3){
      idvec.push_back(mID);
      delete cl;
      continue;
    }
    //---------------
#endif

    //---- Lock all components ----
    for(Int_t k = 0; k < (Int_t) sclvec.size(); k++) sclvec[k]->Lock();
    //-----------------------------

    clmap[cl->GetClusterID()] = cl;

  }

  //----- Unlock failed mother small clusters -----
  for(Int_t i = 0; i < (Int_t) idvec.size(); i++){
      sclmap.find(idvec[i])->second->Unlock();
  }
  //-----------------------------------------------

  delete emutil;
  delete ut;

}

//==========================================================================
//* MakeElectronClusters -----------------------------------------------------

void S4CALEMClusterMaker::MakeElectronClusters(std::map<Int_t,S4CALRealCluster*> &clmap,
					       std::map<Int_t,S4CALSmallCluster*> &sclmap,
					       std::map<Int_t,S4HYBTrack*> &trkmap)
{

  //  static const Double_t calinR = gJSF->Env()->GetValue("J4CAL.BarrelTower.FrontRadius",210);
  //  static const Double_t calinZ = gJSF->Env()->GetValue("J4CAL.EndcapTower.FrontZ",270);
  static const Double_t calinR = S4CALAddress::GetBarrelInnerR();
  static const Double_t calinZ = S4CALAddress::GetBarrelHalfZ();


  S4CALEMClusterMakerUtils *emutil = new S4CALEMClusterMakerUtils();
  S4CALClusterUtils *ut = new S4CALClusterUtils();

  std::map<Int_t, S4CALSmallCluster*>::iterator itr;
  std::multimap<Double_t, S4CALSmallCluster*>::reverse_iterator titr;
  std::vector<Int_t> idvec;

  std::vector<S4CALSmallCluster*> sclv;

  //--- sort sclmap --------
  std::multimap<Double_t,S4CALSmallCluster*> tmpmap;
  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){
    if(itr->second->IsLocked()) continue;
    tmpmap.insert(std::make_pair(itr->second->GetEnergy(),itr->second));
  }
  //------------------------


  //--------------- Make and Add clusters ------------------

  for(titr = tmpmap.rbegin(); titr != tmpmap.rend(); titr++){

    if(titr->second->IsLocked() || !(titr->second->IsEM()) ) continue;

#ifndef __USEHIT__
    if( titr->second->GetNHits() < 5 ) continue;
#endif

    Int_t    mID      = titr->second->GetClusterID();
    TVector3 mpos     = titr->second->GetPosition();
    TVector3 mthrustv = titr->second->GetThrustVector();
    if(mthrustv.Mag() < 1.e-6) mthrustv = mpos*(1./mpos.Mag());


    //--- cut 1: The nearest track distance cut -----
    std::pair<Int_t,Double_t> distpair = S4CALSmallClusterUtils::NearestTrackIs(trkmap,mpos);
    Double_t dist = distpair.second;
    if(dist > 3.){
      titr->second->Lock();
      idvec.push_back(mID);
      continue;
    }
    //-----------------------------------------------

    //------------ get helical track ----------------
    S4HYBTrack* etrack  = trkmap.find(distpair.first)->second;
    TVector3    etrkmom = (S4CALSmallClusterUtils::GetInTrack(etrack,calinR,calinZ))[1];
    Double_t tracke = etrkmom.Mag();

    //----- temporary! -------
    if(tracke < 2.){
      titr->second->Lock();
      idvec.push_back(mID);
      continue;
    }
    //------------------------

    //-----------------------------------------------

    //-------- loose cut ------------
    Double_t ratio1 = TMath::Abs(tracke - titr->second->GetEnergy())/tracke;
    if(ratio1 > 0.5){
      titr->second->Lock();
      idvec.push_back(mID);
      continue;
    }
    //-------------------------------

    //----------- cut 2: EHCompare cut --------------
    (titr->second->GetPFOMap()).erase(kgEHCompare);
    std::vector<Int_t> idlist;
    idlist.clear();
    idlist = emutil -> EHCompare(sclmap,
				 titr->second,
				 fEHCompareTubeR,       // tubeR
				 fEHCompareTubeHalfH);  // tubeH

    if( idlist.size() > 0){
      Double_t ratioeh = (titr->second->GetPFOMap().find(kgEHCompare)->second)[1];
      if(ratioeh > fEHCompareHERatioCut){
	titr->second->Lock();
	idvec.push_back(mID);
	continue;
      }
    }else{

      titr->second->Lock();
      idvec.push_back(mID);
      continue;

    }
    //-----------------------------------------------

    //------- cut 3: EMLongitudinal cut -------------
    std::vector<Int_t> longitudinal; 
    longitudinal.clear();
    Double_t depth = 1.e+5;
    longitudinal = emutil->EMLongitudinalShape(sclmap, titr->second, 5.);
    if(longitudinal.size() > 0){

      depth = (titr->second->GetPFOMap().find(kgEMLongitudinal)->second)[1];
      if(depth > 15.){
	titr->second->Lock();
	idvec.push_back(mID);
	continue;
      }

    }else{

      titr->second->Lock();
      idvec.push_back(mID);
      continue;

    }
    //-----------------------------------------------

    //------- cut 4: EMFitter Chi-square cut --------
    (titr->second->GetPFOMap()).erase(kgEMFitter);
    std::vector<Int_t> vec = emutil->EMFitter(sclmap,
					      titr->second,
					      fFitterTubeRCut,
					      fFitterTubeHalfHCut);
    if( vec.size() > 0 ){

      Double_t chi2 = (titr->second->GetPFOMap().find(kgEMFitter)->second)[3];
      //if( chi2 > fChisquareCut || chi2 < 1.e-2){
      if( chi2 > 1.2 || chi2 < 1.e-2){
#if 0
	// Lock all checked small clusters
	for(Int_t i = 0; i < (Int_t)vec.size(); i++ ){
	  sclmap.find(vec[i])->second->Lock();
	  idvec.push_back(vec[i]);
	}
#else
	// Lock only mother small cluster
	titr->second->Lock();
	idvec.push_back(mID);
#endif
	continue;
      }

    }else{

      titr->second->Lock();
      idvec.push_back(mID);
      continue;

    }
    //-----------------------------------------------


    //------------ make a new real cluster ----------------
    S4CALRealCluster *cl = new S4CALRealCluster(trkmap.find(distpair.first)->second);
    
    //--- add small clusters within small tube ----
    for(itr = sclmap.begin(); itr != sclmap.end(); itr++){

      if(itr->second->IsLocked()) continue;
	
      //if( ut->IsWithinTube(itr->second->GetPosition(),mpos,mthrustv,fTubeRCut,fTubeHalfHCut) ){
      if( ut->IsWithinTube(itr->second->GetPosition(),mpos,etrkmom,fTubeRCut,fTubeHalfHCut) ){
	cl -> Add(itr->second);
	sclv.push_back(itr->second);
	itr->second->Lock();
      }
    }

    cl -> SetClusterType(kgEmCharged_cl);
    cl -> SetParticleType(kgElectron_p);
    cl -> Update();
    //---------------------------------------------


    //---- cut 5: MaxELayer and energy matching cut  -----
    Double_t cle    = cl->GetEnergy();
    Double_t comp   = TMath::Abs(tracke - cle)/tracke;

    if((cl->GetMaxELayerID()) >= 19 || comp > 0.15){
      for(Int_t l = 0; l < (Int_t) sclv.size(); l++){
	sclv[l]->Unlock();
      }
      titr->second->Lock();
      idvec.push_back(mID);
      delete cl;
      continue;
    }else{
      etrack->Lock();
    }
    //----------------------------------------------------
    
    clmap.insert(std::pair<Int_t, S4CALRealCluster*>(cl->GetClusterID(), cl));

  }

  //--------------------------------------------------------

  for(Int_t i = 0; i < (Int_t) idvec.size(); i++){
    sclmap.find(idvec[i])->second->Unlock();
  }
  
  delete emutil;
  delete ut;
  
}
