//*************************************************************************
//* ---------------------
//* S4CALNHDClusterMaker Class
//* ---------------------
//* (Description)
//*
//* (Requires)
//*
//* (Provides)
//*
//* (Update Record)
//*
//*************************************************************************
#include "S4CALClusterUtils.h"
#include "S4CALRealClusterMaker.h"
#include "S4CALSmallClusterUtils.h"
#include "S4CALNHDClusterMaker.h"
#include "S4CALAddress.h"

#include "JSFSteer.h"
#include "TNtuple.h"
#include <iostream>

#include "S4CALHitMaker.h"

#define __LDC__
//#define __LIKELIHOOD__

ClassImp(S4CALNHDClusterMaker)


//=====================================================================
//* constructor -------------------------------------------------------
S4CALNHDClusterMaker::S4CALNHDClusterMaker()
{
}


//=====================================================================
//* destructor --------------------------------------------------------

S4CALNHDClusterMaker::~S4CALNHDClusterMaker()
{
}

//=====================================================================
//* MakeNHDClusters ---------------------------------------------------

void S4CALNHDClusterMaker::MakeNHDClusters(std::map<Int_t,S4CALRealCluster*> &clmap,
					   std::map<Int_t,S4CALHit*> &hitmap)
{

  std::vector<S4CALSmallCluster*> sclvec;

  std::map<Int_t,S4CALHit*>::iterator hitr;
  std::map<Int_t,S4CALHit*> hitmap2;

  for(hitr = hitmap.begin(); hitr != hitmap.end(); hitr++){
    if(hitr->second->IsLocked()) continue;
    hitmap2.insert(*hitr);
  }

#ifdef __LDC__
  //* LDC group's clustering

  //------ make small clusters -------
  // ref. LDC's parameter = (2., 10., 10., 50., 2., 10., 5., 5.)
#if 0
  //* ECFA
  S4CALSmallClusterUtils::SetEMRCutMin(2.);
  S4CALSmallClusterUtils::SetEMRCutMax(14.);
  S4CALSmallClusterUtils::SetHDRCutMin(10.);
  S4CALSmallClusterUtils::SetHDRCutMax(50.);
  S4CALSmallClusterUtils::SetEMDCut(3.);
  S4CALSmallClusterUtils::SetHDDCut(7.);
  S4CALSmallClusterUtils::SetEMNSteps(5);
  S4CALSmallClusterUtils::SetHDNSteps(5);
#else
  //* thesis
  S4CALSmallClusterUtils::SetEMRCutMin(2.);
  S4CALSmallClusterUtils::SetEMRCutMax(14.);
  S4CALSmallClusterUtils::SetHDRCutMin(10.);
  S4CALSmallClusterUtils::SetHDRCutMax(50.);
  S4CALSmallClusterUtils::SetEMDCut(8.);
  S4CALSmallClusterUtils::SetHDDCut(12.);
  S4CALSmallClusterUtils::SetEMNSteps(5);
  S4CALSmallClusterUtils::SetHDNSteps(5);
#endif

  std::vector<S4CALSmallCluster*> tmpsvec;
  // this method doesn't care about if hit is locked or not
  tmpsvec = S4CALSmallClusterUtils::MakeClustersLDC(hitmap2, kFALSE);
  //-----------------------------------

  //------ merge small clusters -------
  // ref. LDC's parameter = (10., 5., 50., 10., 15., -1)
  S4CALSmallClusterUtils::SetEMDistCutBack(10.); // cm
  S4CALSmallClusterUtils::SetEMDistCutFwd(5.);  // cm
  S4CALSmallClusterUtils::SetHDDistCutBack(50.); // cm
  S4CALSmallClusterUtils::SetHDDistCutFwd(15.); // cm
  S4CALSmallClusterUtils::SetNHitsCut(25);
  S4CALSmallClusterUtils::SetMinLayCut(-1);
  
  sclvec  = S4CALSmallClusterUtils::MergeSmallClustersLDC(tmpsvec, kFALSE);
  //-----------------------------------

//#elif 1

  //------ simple small clustering (e-dep information is not used) -------

  std::vector<S4CALSmallCluster*> tmpsclv;

  //static Int_t nemmc = gJSF->Env()->GetValue("J4CAL.EM.MiniCone.NClones",3);
  //static Int_t nemmt = gJSF->Env()->GetValue("J4CAL.EM.MiniTower.NClones",3);
  //static Int_t nhdmc = gJSF->Env()->GetValue("J4CAL.HD.MiniCone.NClones",1);
  //static Int_t nhdmt = gJSF->Env()->GetValue("J4CAL.HD.MiniTower.NClones",1);

  S4CALHit *firstcell = 0;
  
  std::vector<S4CALHit*> hitv = S4CALSmallClusterUtils::SortHitMap(hitmap2);
  
  // if second is true, it is already used as a center cell
  std::map<Int_t, Bool_t> usedCell;
  
  for(Int_t n = 0; n < (Int_t) hitv.size(); n++){
    
    if(hitv[n]->IsLocked()) continue;
    
    Int_t cellID = 0;
    firstcell = hitv[n];
    
    S4CALSmallCluster *smallcls = new S4CALSmallCluster();
    
    cellID = firstcell->GetCellID();
    usedCell.clear();
    usedCell.insert(std::pair<Int_t, Bool_t>(cellID, kTRUE));
    smallcls -> Add(firstcell); // Add hit into a Small Cluster
    firstcell->Lock();
    
    Bool_t isEM       = firstcell->IsEM();
    Bool_t isBarrel   = (firstcell->GetCALType() & ( kgBaEm | kgBaHd ));
    TVector3 firstpos = firstcell->GetCellPosition();  
    
    //-------------------------------------------
    // add neighbor cells
    //-------------------------------------------
    
    while ( cellID > 0 ) {

      usedCell[cellID] = kTRUE;  // if true, it is already used as a center
      
      //-------------------------------------
      // add neighbor cells to small cluster
      //-------------------------------------
      //* Find neighbor cells
      std::vector<Int_t> neighborVector = S4CALAddress::GetListOfNeighbors( cellID, isEM, isBarrel );
      for ( Int_t i = 0; i < (Int_t)neighborVector.size(); i++ ) {

	Int_t neighborcell = neighborVector[i];
	
	std::map<Int_t,S4CALHit*>::iterator hitr = hitmap.find(neighborcell);

	if( (hitmap.count(neighborcell) > 0) && !(hitr->second->IsLocked()) && ((hitr->second->IsEM()) == isEM ) ){
	  smallcls -> Add(hitr->second); // Add hit into a Small Cluster
	  usedCell.insert( std::pair<Int_t, Bool_t>(neighborcell, kFALSE) );
	  hitr->second->Lock();
	}
      }

      //--- check small cluster nhits -----
      smallcls->Update();
      if( (smallcls->GetNHits()) > (smallcls->GetMaxNHitsCut()) )  break;
      //-----------------------------------    
      
      //--- find next center ---------------
      std::map<Int_t, Bool_t>::iterator itr;
      cellID = 0;
      Double_t tmpdist = 1.e+5;
      for(itr = usedCell.begin(); itr != usedCell.end(); itr++ ){
	if( !(itr -> second) ){
	  TVector3 tmppos = hitmap.find(itr->first)->second->GetCellPosition();
	   if( (firstpos-tmppos).Mag() < tmpdist ){ // choose the nearest one
	     cellID = itr -> first;
	     tmpdist = (firstpos - tmppos).Mag();
	   }
	   
	}
       }
      //------------------------------------
      
    } // end of while(cellID > 0) loop

    tmpsclv.push_back(smallcls);

  } // end of for(Int_t n = 0; ...) loop
  //-----------------------------------------------------------------------

  //---------------------------- merging ----------------------------------
  std::multimap<Int_t,S4CALSmallCluster*> nmap;
  std::multimap<Int_t,S4CALSmallCluster*>::reverse_iterator nitr;

  for(Int_t k = 0; k < (Int_t) tmpsclv.size(); k++){
    if(tmpsclv[k]->IsLocked()) continue;
    nmap.insert(std::pair<Int_t,S4CALSmallCluster*>(tmpsclv[k]->GetNHits(),tmpsclv[k]));
  }

  Double_t dx = 0.;
  Double_t dt = 0.;

  for(nitr = nmap.rbegin(); nitr != nmap.rend(); nitr++){

    Bool_t match = kFALSE;

    (nitr->second->IsEM()) ? (dx = 20., dt = 5.) : (dx = 40, dt = 10.);

    for(Int_t k = 0; k < (Int_t) sclvec.size(); k++){

      TVector3 center = sclvec[k]->GetPosition();
      TVector3 sclpos = nitr->second->GetPosition();
      Double_t ctof   = sclvec[k]->GetTOF();
      Double_t scltof = nitr->second->GetTOF();
      if( ((sclpos-center).Mag() < dx) && (TMath::Abs(scltof-ctof) < dt) ){
        sclvec[k]->Add(nitr->second);
        match = kTRUE;
        break;
      }

    }

    if(!match)  sclvec.push_back(nitr->second);

  }
  //-----------------------------------------------------------------------


#elif 1
  //* dX, dT clustering
  std::vector<S4CALSmallCluster*> tmpsvec;
  for(hitr = hitmap2.begin(); hitr != hitmap2.end(); hitr++){
    hitr->second->Lock();
    S4CALSmallCluster* scl = new S4CALSmallCluster();
    scl->Add(hitr->second);
    scl->Update();
    tmpsvec.push_back(scl);
  }

   S4CALSmallClusterUtils::SetEMdX(20.);
   S4CALSmallClusterUtils::SetHDdX(30.);
   S4CALSmallClusterUtils::SetEMdT(20.);
   S4CALSmallClusterUtils::SetHDdT(40.);

  sclvec = S4CALSmallClusterUtils::MergeSmallClustersdXdT(tmpsvec,kFALSE,"");

#else
  //* do not perform the clustering
  for(hitr = hitmap2.begin(); hitr != hitmap2.end(); hitr++){
    hitr->second->Lock();
    S4CALSmallCluster* scl = new S4CALSmallCluster();
    scl->Add(hitr->second);
    scl->Update();
    sclvec.push_back(scl);
  }
#endif


  //--------- distinguish CHD/gamma clusters -----------
#ifdef __LIKELIHOOD__
  static const Int_t nparams  = 4;
  static const Int_t mbgkinds = 1;
  static TMatrixD sprob(nparams,1);
  static TMatrixD bgprob(nparams,mbgkinds);
  static Bool_t first = kTRUE;

  //-------- sample histograms ---------
  static std::map<Double_t,Double_t> nhdv;
  static std::map<Double_t,Double_t> gamv;
  static std::map<Double_t,Double_t> ev;
  static std::map<Double_t,Double_t> chdv;
  static std::map<Double_t,Double_t> nhdd;
  static std::map<Double_t,Double_t> gamd;
  static std::map<Double_t,Double_t> ed;
  static std::map<Double_t,Double_t> chdd;
  static std::map<Double_t,Double_t> nhdn;
  static std::map<Double_t,Double_t> chdn;
  static std::map<Double_t,Double_t> nhdm;
  static std::map<Double_t,Double_t> chdm;
  //------------------------------------

  //---------- make sample histograms -----------
  if(first){
    first = kFALSE;
    TDirectory *last = gDirectory;
    last->cd();
    Int_t nzqqbar = 11;
    Char_t fzqqbar[11][512];
    TChain *tree = new TChain("rcltree");
  for(Int_t i=1;i<nzqqbar;i++){
    sprintf(fzqqbar[i],"/data4/jlc/tyosioka/jupiter/cvs.060614a/zqqbar/nt/uds91-jun06_2x2_2m-500-gamlike-%d.root",i);
    if (i!=17) {
        tree->Add(fzqqbar[i]);
      }
    }

    nhdv = 
      S4CALClusterUtils::MakeSampleHistogram(tree,"v","((isnhd || isnotdecided))*(en+ek)",30,30.);
    //    gamv = S4CALClusterUtils::MakeSampleHistogram(tree,"v","((isnhd || isnotdecided) && v < 38.)*(egam)",50);
    chdv 
      = S4CALClusterUtils::MakeSampleHistogram(tree,"v","((isnhd || isnotdecided))*(epi+ep+eck+eother+emu)",30,30.);

    nhdd 
      = S4CALClusterUtils::MakeSampleHistogram(tree,"edensity","(isnhd || isnotdecided)*(en+ek)",30,0.000003);
    //    gamd = S4CALClusterUtils::MakeSampleHistogram(tree,"TMath::Log(edensity)","((isnhd || isnotdecided) && edensity > 0.)*(egam)",50);
    chdd 
      = S4CALClusterUtils::MakeSampleHistogram(tree,"edensity","(isnhd || isnotdecided)**(epi+ep+eck+eother+emu)",30,0.000003);


    nhdn 
      = S4CALClusterUtils::MakeSampleHistogram(tree,"edep/nhits","(isnhd || isnotdecided)*(en+ek)",30,0.006);
    //    gamd = S4CALClusterUtils::MakeSampleHistogram(tree,"TMath::Log(edensity)","((isnhd || isnotdecided) && edensity > 0.)*(egam)",50);
    chdn 
      = S4CALClusterUtils::MakeSampleHistogram(tree,"edep/nhits","(isnhd || isnotdecided)*(epi+ep+eck+eother+emu)",30,0.006);


    nhdm 
      = S4CALClusterUtils::MakeSampleHistogram(tree,"meanlayerID","(isnhd || isnotdecided)*(en+ek)",30,90.);
    //    gamd = S4CALClusterUtils::MakeSampleHistogram(tree,"TMath::Log(edensity)","((isnhd || isnotdecided) && edensity > 0.)*(egam)",50);
    chdm 
      = S4CALClusterUtils::MakeSampleHistogram(tree,"meanlayerID","(isnhd || isnotdecided)*(epi+ep+eck+eother+emu)",30,90.);

  }
  //--------------------------------------------

  /////////////////////////////////////////////////////////////
  static TNtuple* like = new TNtuple("like","likelihood test","e:en:ek:echd:egam:ee:eother:l");
  /////////////////////////////////////////////////////////////

#endif
  
  for(Int_t k = 0; k < (Int_t) sclvec.size(); k++){
    sclvec[k]->Unlock();
    Double_t v = ((sclvec[k]->GetPosition()).Mag())/(sclvec[k]->GetTOF());
    Double_t edensity = S4CALSmallClusterUtils::CalcEnergyDensity(hitmap2,sclvec[k]->GetPosition(),20.);
    Double_t edep = sclvec[k]->GetEnergyDeposit();
    Int_t nhits = sclvec[k]->GetNHits();
    Double_t meanlayerID = sclvec[k]->GetMeanLayerID();

#ifdef __LIKELIHOOD__
    
    sprob(0,0)  = S4CALClusterUtils::CalcProbability(v,nhdv);
    sprob(1,0)  = S4CALClusterUtils::CalcProbability(edensity,nhdd);
    sprob(2,0)  = S4CALClusterUtils::CalcProbability(edep/nhits,nhdn);
    sprob(3,0)  = S4CALClusterUtils::CalcProbability(meanlayerID,nhdm);

    bgprob(0,0) = S4CALClusterUtils::CalcProbability(v,chdv);
    bgprob(1,0) = S4CALClusterUtils::CalcProbability(edensity,chdd);
    bgprob(2,0) = S4CALClusterUtils::CalcProbability(edep/nhits,chdn);
    bgprob(3,0) = S4CALClusterUtils::CalcProbability(meanlayerID,chdm);

    //    std::cout << "Signal Probability : " << sprob(0,0) << " " << sprob(1,0) << " "
    //	      << sprob(2,0) << " " << sprob(3,0) << std::endl;
    //    std::cout << "Background Probability : " << bgprob(0,0) << " " << bgprob(1,0) << " "
    //	      << bgprob(2,0) << " " << bgprob(3,0) << std::endl;
    //    if(v >= 30.){ // v is in the out of range
    //      sprob(0,0)  = 0.5;
    //      bgprob(0,0) = 0.5;
    //      bgprob(0,1) = 0.5;
    //      bgprob(0,2) = 0.5;
    //    }

    Double_t likelihood = S4CALClusterUtils::CalcLikelihood(nparams,mbgkinds,sprob,bgprob);

    ///////////////////////// Tree /////////////////////////////
    Int_t    pid = 0;
    Double_t e      = sclvec[k]->GetEnergyDeposit();
    Double_t en     = 0.;
    Double_t ek     = 0.;
    Double_t echd   = 0.;
    Double_t egam   = 0.;
    Double_t ee     = 0.;
    Double_t eother = 0.;
    std::map<Int_t, Double_t>::iterator piditr;
    for(piditr = (sclvec[k]->GetPIDMap()).begin(); piditr != (sclvec[k]->GetPIDMap()).end(); piditr++){
      
      pid = piditr->first;
      if(TMath::Abs(pid) == 211)       echd += piditr->second;
      else if(TMath::Abs(pid) == 22)   egam += piditr->second;
      else if(TMath::Abs(pid) == 11)   ee   += piditr->second;
      else if(TMath::Abs(pid) == 130)  ek   += piditr->second;
      else if(TMath::Abs(pid) == 2212) echd += piditr->second;
      else if(TMath::Abs(pid) == 2112) en   += piditr->second;
      else if(TMath::Abs(pid) == 321)  echd += piditr->second;
      else  eother += piditr->second;
    }

    like->Fill(e,en,ek,echd,egam,ee,eother,likelihood);

    std::cout << "likelihood = " << likelihood
	      << ", nhd fraction = " << (en+ek)/e << std::endl;
    ////////////////////////////////////////////////////////////////

    if(likelihood < 0.73) sclvec[k]->Lock(); // satellite-like

#else
    
    ///////////////////////////////////////////////////////////////////
    //* def.
#if 1
    // default
    if(v < 12. || edensity < 5.e-7) sclvec[k]->Lock(); // satellite-like
#else
    // satellites 2
    if(v < 15. && edensity < 4.e-7) sclvec[k]->Lock(); // satellite-like
#endif
    ///////////////////////////////////////////////////////////////////
#endif
  }
  //----------------------------------------------------

  for(Int_t k = 0; k < (Int_t) sclvec.size(); k++){
    S4CALRealCluster *cl = new S4CALRealCluster();
    cl->Add(sclvec[k]);
    if(sclvec[k]->IsLocked()) cl->SetParticleType(kgNotDecided_p);
    else                      cl->SetParticleType(kgHdNeutral_p);
    cl->Update();
    clmap.insert(std::pair<Int_t,S4CALRealCluster*>(cl->GetClusterID(),cl));
    delete sclvec[k];
  }

  /*
  //------ add satellite like hits into hitmap -------                 
  S4CALHitMaker *chm=(S4CALHitMaker*)gJSF->FindModule("S4CALHitMaker");
  S4CALHitBuf   *chb=(S4CALHitBuf*)chm->EventBuf();
  //  S4CALHitBuf   *chb=(S4CALHitBuf*)GetInputDataBufPtr();
  TObjArray     *chts=chb->GetOutputBufPtr(); 
  TIter next(chts);  
  S4CALHit *calhitp = 0;
  Double_t hx = 0.;
  Double_t hy = 0.;
  Double_t hz = 0.;
  Double_t ht = 0.;
  Double_t hv = 0.;
  while( (calhitp = (S4CALHit*)next()) ){
    hx = calhitp->GetCellPosition()[0];
    hy = calhitp->GetCellPosition()[1];
    hz = calhitp->GetCellPosition()[2];
    ht = calhitp->GetTOF();                
    hv = TMath::Sqrt(hx*hx+hy*hy+hz*hz)/ht;
    if ( (hv > 18.) ) continue;
    S4CALRealCluster  *cl  = new S4CALRealCluster();
    cl->Add(calhitp);
    cl->SetParticleType(kgNotDecided_p);
    cl->Update();
    clmap.insert(std::pair<Int_t,S4CALRealCluster*>(cl->GetClusterID(),cl));
  }                                                                        
  */
  //--------------------------------------------------      
}

//=====================================================================
//* DefiniteNHDIs -----------------------------------------------------

std::vector<Int_t> S4CALNHDClusterMaker::DefiniteNHDIs(std::map<Int_t,S4CALSmallCluster*> &sclmap, std::map<Int_t,S4HYBTrack*> &trkmap)
{

  std::vector<Int_t> idvec;

  // trk distance, velocity, e/nhits, minlayerID (for ECAL scls)


  return idvec;

}
