//*************************************************************************
//* ---------------------
//* S4ITTrack Class
//* ---------------------
//* (Description)
//*	Class to store "S4ITTrackHit"s for a single track.
//* (Requires)
//*	TObjArray
//* (Provides)
//*	S4ITTrack		: Array containing "S4ITTrackHit"s
//* (Update Record)
//*	2004/08/26  A.Yamaguchi    Original Ver.
//*************************************************************************

#include <iostream>
#include <iomanip>
#include "S4ITTrack.h"
#include "S4ITTrackHit.h"
#include "TPolyMarker3D.h"
#include "TVirtualPad.h"
//#define __DEBUG__

ClassImp(S4ITTrack)

//=====================================================================
//* constructor -------------------------------------------------------

S4ITTrack::S4ITTrack()
{
#ifdef __DEBUG__
   std::cerr << "constructor for S4ITTrack is called..." << std::endl;
#endif
   SetOwner();
}

S4ITTrack::S4ITTrack(S4ITTrackMaker *trkm)
         : U4ITTrack(trkm)
{
#ifdef __DEBUG__
   std::cerr << "constructor for S4ITTrack is called..." << std::endl;
#endif
   SetOwner();
}

//=====================================================================
//* destructor --------------------------------------------------------

S4ITTrack::~S4ITTrack()
{
}

//=====================================================================
//* GetPDGCode --------------------------------------------------------

Int_t S4ITTrack::GetPDGCode()
{
   S4ITTrackHit *thp = (S4ITTrackHit *)((TObjArray *)this)->operator[](0);
   S4ITHit   *php = (S4ITHit *)thp->GetHitPtr();

   return php->GetExactHitPtr()->GetPDGCode();
}

//=====================================================================
//* GetTrackID --------------------------------------------------------

Int_t S4ITTrack::GetTrackID()
{
   S4ITTrackHit *thp = (S4ITTrackHit *)((TObjArray *)this)->operator[](0);
   S4ITHit   *php = (S4ITHit *)thp->GetHitPtr();
#if 0
   std::cerr << "tptrk = " << (void *)this << " j4trkid = " << php->GetExactHitPtr()->GetTrackID() << std::endl;
#endif

   return php->GetExactHitPtr()->GetTrackID();
}

//=====================================================================
//* GetMotherTrackID --------------------------------------------------

Int_t S4ITTrack::GetMotherTrackID()
{
   S4ITTrackHit *thp = (S4ITTrackHit *)((TObjArray *)this)->operator[](0);
   S4ITHit   *php = (S4ITHit *)thp->GetHitPtr();

   return php->GetExactHitPtr()->GetMotherTrackID();
}

//=====================================================================
//* Draw --------------------------------------------------------------

void S4ITTrack::Draw(Int_t color, const Char_t * /* opt */)
{
   if (!gPad) return;
   gPad->cd();

   TPolyMarker3D *pm3dp = new TPolyMarker3D(this->GetEntries());
   pm3dp->SetBit(kCanDelete);
   pm3dp->SetMarkerColor(color);

   Int_t nhits = 0;
   TIter next(this);
   S4ITTrackHit *trkhitp = 0;
   while ((trkhitp = static_cast<S4ITTrackHit *>(next()))) { 
      TVector3 pos = trkhitp->GetHitPtr()->GetExactHitPtr()->GetPosition();
      pm3dp->SetPoint(nhits, pos.X(), pos.Y(), pos.Z());
      nhits++;
   }
   pm3dp->Draw();
   gPad->Update();
}

//=====================================================================
//* Draw --------------------------------------------------------------

void S4ITTrack::DebugPrint(const Char_t *)
{
}

//=====================================================================
//* Verify ------------------------------------------------------------

Bool_t S4ITTrack::Verify()
{
   if (!GetEntries()) return kFALSE;

   S4ITTrackHit::SetSortBy("Time");
   UnSort(); 
   Sort(); 

   if (GetEntries() > 1) {
      TIter next(this);
      Int_t lyr1 = static_cast<S4ITTrackHit *>
                  (next())->GetHitPtr()->GetExactHitPtr()->GetLayerID();
      Int_t lyr2 = static_cast<S4ITTrackHit *>
                  (next())->GetHitPtr()->GetExactHitPtr()->GetLayerID();
      if (lyr2 < lyr1) return kFALSE;
      next.Reset();

      S4ITTrackHit *thitp = 0;
      Int_t last = -999;
      while ((thitp = static_cast<S4ITTrackHit *>(next()))) {
         S4ITExactHit *ehitp = thitp->GetHitPtr()->GetExactHitPtr();
         Int_t cur = ehitp->GetLayerID();
         if (cur <= last) {
            thitp->Lock();
            break;
         }
         last = cur;
      }
      while ((thitp = static_cast<S4ITTrackHit *>(next()))) thitp->Lock();
   }
   return kTRUE;
}

