//*************************************************************************
//* -----------
//* S4PFOAnalysis
//* -----------
//* (Description)
//* 	Clasess to analyze PFO data.
//* (Update Record)
//*	2004/11/06  S.Yamamoto	Original version.
//*************************************************************************

#include "JSFSteer.h"
#include "S4PFOAnalysis.h"
#include <iostream>
#include <sstream>

#include "S4PFOMakerBuf.h"
#include "TF1.h"
ClassImp(S4PFOAnalysis)

//___________________________________________________________________________
S4PFOAnalysis::S4PFOAnalysis(const Char_t *name, const Char_t *title)
             : JSFModule(name, title)
{
   //std::cerr << " constructor for S4PFOAnalysis is called..." << std::endl;
}

//___________________________________________________________________________
S4PFOAnalysis::~S4PFOAnalysis()
{
}

//___________________________________________________________________________
Bool_t S4PFOAnalysis::Initialize()
{
   TDirectory *last = gDirectory;
   gFile->cd("/");

   fHPFOCharge  = new TH1D("PFOCharge", "", 500, -2, 2);
   fHPFOEclEtk  = new TH2D("PFOEclEtk", "", 500, 0., 10., 500, 0., 10.);
   fHPFOEposEtk = new TH2D("PFOEposEtk", "", 500, 0., 10., 500, 0., 10.);
   fHPFOEclEpos = new TH2D("PFOEclEpos", "", 500, 0., 10., 500, 0., 10.);
   fHPFOPidEclEtk = new TH2D("PFPidOEclEtk", "", 500, 0., 10., 500, 0., 10.);
   fHPFOPid     = new TH1D("PFOPid", "", 12001, -6000., 6000.);
   fHPFONo      = new TH1D("PFONo", "", 500, 0., 100.);
   fHPFOE       = new TH1D("PFOEpfo", "Reconstructed PFOEnergy", 500, 0., 10.);
   fHPFOEnd     = new TH1D("PFOEnd", "", 500, 0., 10.);
   fHPFOEe      = new TH1D("PFOEe", "", 500, 0., 10.);
   fHPFOEmu     = new TH1D("PFOEmu", "", 500, 0., 10.);
   fHPFOEg      = new TH1D("PFOEg", "", 500, 0., 10.);
   fHPFOEpi     = new TH1D("PFOEpi", "", 500, 0., 10.);
   fHPFOEkl     = new TH1D("PFOEkl", "", 500, 0., 10.);
   fHPFOEpr     = new TH1D("PFOEpr", "", 500, 0., 10.);
   fHPFOEnu     = new TH1D("PFOEnu", "", 500, 0., 10.);
#if 0
   fHClusterEne = new TH1D("ClusterEne", "", 500, 0, 60);
   fHPreHitEne  = new TH1D("PreHitEne", "", 500, 0, 5.5);
   fHEneRatio   = new TH1D("EneRatio", "", 500, -4, 4);
   fHTrackEne   = new TH1D("TrackEne", "", 500, 0, 10);
   fHEneRatiotk = new TH1D("EneRatiotk", "", 500, 0.9, 2.);
   fHDistDiff   = new TH1D("DistDiff", "", 500, -0.9, 10.);
#endif
   last->cd();

   return kTRUE;
}


//___________________________________________________________________________
Bool_t S4PFOAnalysis::Process(Int_t ev)
{
   MonitorPFOs(ev);
#if 1
   std::cerr << "Analysis::Process evt " << ev << " is finished" << std::endl;
#endif
   return kTRUE;   
}   

//___________________________________________________________________________
void S4PFOAnalysis::MonitorPFOs(Int_t ev)
{
   std::cerr << "ev = " << ev << std::endl;
   TDirectory *last = gDirectory;
   gFile->cd("/");

   TObjArray *outputbufp = dynamic_cast<S4PFOMakerBuf *> 
     (gJSF->FindModule("S4PFOMaker")->EventBuf())->GetOutputBufPtr();

   TIter nextpfo(outputbufp);
   S4PFObject *pfop;
   Int_t Npfo = outputbufp->GetEntries();
   fHPFONo->Fill(Npfo, 1.);
   Double_t Epfo;
   Double_t EPFO;

   EPFO = 0.;

   while ((pfop = dynamic_cast<S4PFObject *> (nextpfo()))) {
     Double_t Cpfo = pfop->GetCharge();
     fHPFOCharge->Fill(Cpfo, 1.);
     Int_t    PIDpfo = pfop ? pfop->GetPDGCode() : 0;
     fHPFOPid->Fill(PIDpfo, 1.);
     Int_t PID = pfop->GetPID();
     S4VExactHit *posp = pfop->GetExactHitPtr();
     Double_t Epos = posp ? posp->GetTotalEnergy() : 0.;
     S4CALCluster *clp  = pfop->GetClusterPtr();
     Double_t Ecl = clp ? clp->GetEnergy() : 0.;
     S4HYBTrack   *htkp = pfop->GetHYBTrackPtr();
     Double_t Etk = htkp ? pfop->E() : 0.;
     fHPFOEclEtk->Fill(Ecl, Etk);
     fHPFOEposEtk->Fill(Epos, Etk);
     fHPFOEclEpos->Fill(Ecl, Epos);
     fHPFOPidEclEtk->Fill(Ecl, Etk, PIDpfo);
     Epfo = 0.;
     Epfo = htkp ? Etk : Ecl;
    
     //e,gammma
     Double_t End = 0.;
     Double_t Ee  = 0.;
     Double_t Emu = 0.;
     Double_t Eg  = 0.;
     Double_t Epi = 0.;
     Double_t Ekl = 0.;
     Double_t Epr = 0.;
     Double_t Enu = 0.;

     switch (PID) {
        case 0: End = htkp ? htkp->GetEnergy() : clp->GetEnergy();
                fHPFOEnd->Fill(End, 1.);
                break;
        case 1: Ee  = htkp ? htkp->GetEnergy() : 0.;
                fHPFOEe->Fill(Ee, 1.);
                break;
        case 2: Emu = htkp ? htkp->GetEnergy() : 0.;
                fHPFOEmu->Fill(Emu, 1.);
                break;
        case 3: Eg = clp ? clp->GetEnergy() : 0.;
                fHPFOEg->Fill(Eg, 1.);
                break;
        case 4: Epi = htkp ? htkp->GetEnergy() : 0.;
                fHPFOEpi->Fill(Epi, 1.);
                break;
        case 5: Ekl = clp  ? clp->GetEnergy() : 0.;
                fHPFOEkl->Fill(Ekl, 1.);
                break;
        case 6: Epr = htkp  ? htkp->GetEnergy() : 0.;
                fHPFOEpr->Fill(Epr, 1.);
                break;
        case 7: Enu = clp  ? clp->GetEnergy() : 0.;
                fHPFOEnu->Fill(Enu, 1.);
                break;
     default:break;
 }

#if 0
     fHTrackEne->Fill(Etk, 1.);   
     fHEneRatiotk->Fill(Eratiotk, 1.);   
     //////  track-cluster matching  //////
     JSFRPhiZ xexp; //expected hit position  
     if (htkp) {
        TVector3 xvact = clp->GetPosition();
        htkp->GetHelicalTrackPtr("Kalman")->IntersectWithCylinder(JSFRPhiZ(xvact.Perp(), xvact.Phi(), xvact.Z()), xexp);
	TVector3 xvexp(xexp.XYZ().X(),xexp.XYZ().Y(),xexp.XYZ().Z()); 
	Double_t DistDiff = (clp->GetPosition()-xvexp).Mag();
	using namespace std;
	cerr << "cluster = ("
	     << xvact.Perp() << ","
	     << xvact.Phi() << ","
             << xvact.Z() << ")" << endl;
	  cerr << "track = ("
               <<xvexp.Perp() <<  " "
               <<xvexp.Phi() <<  " "
               <<xvexp.Z() <<  " )" << endl;
	cerr << "clp->GetPosition() = ("
	     << xvact.X() << ","
	     << xvact.Y() << ","
             << xvact.Z() << ")" << endl;
	  cerr << "tracl = ("
               <<xvexp.X() <<  " "
               <<xvexp.Y() <<  " "
               <<xvexp.Z() <<  " )" << endl;
     fHDistDiff->Fill(DistDiff, 1.);

     }
#endif
  
    
   EPFO += Epfo;    
   }
   fHPFOE->Fill(EPFO, 1.);    
   last->cd();
}

//___________________________________________________________________________
#if 0
void S4PFOAnalysis::Draw(Option_t/* *opt */)
{
   gFile->cd("/");
   gDirectory->cd();
}
#endif
