//*************************************************************************   
//* ---------------------                      
//* S4KinkFinder Class
//* ---------------------   
//* (Description) 
//*    Finds kinks and locks their daughters.
//* (Requires)
//*     S4CALCluster, S4HYBTrack
//* (Provides)  
//*     S4KinkFinder            : made object class
//* (Update Record)
//*     2007/01/23  K.Fujii     Original version.
//*************************************************************************

#include "S4KinkFinder.h"
#include "S4PFOMakerBuf.h"
#include "S4HYBTrackMakerBuf.h"

#include <iostream>

#define __DEBUG__

ClassImp(S4KinkFinder)

//=====================================================================
//* constructor -------------------------------------------------------

S4KinkFinder::S4KinkFinder()
{
#ifdef __DEBUG__
   cerr << "S4KinkFinder::S4KinkFinder() :::::::::::::::: " << endl
        << "   Kink Finder created : " << this              << endl;
#endif
}

//=====================================================================
//* destructor --------------------------------------------------------

S4KinkFinder::~S4KinkFinder()
{
}

//=====================================================================       
//* FindKinks -----------------------------------------------------------        

void S4KinkFinder::FindKinks(Int_t scheme)
{
   S4PFOMakerBuf *pfmbp = static_cast<S4PFOMakerBuf *>(
          static_cast<S4PFOMaker *>(gJSF->FindModule("S4PFOMaker"))->EventBuf());
   S4HYBTrackMakerBuf *htmbp = static_cast<S4HYBTrackMakerBuf *>(
          static_cast<S4HYBTrackMaker *>(gJSF->FindModule("S4HYBTrackMaker"))->EventBuf());

   TIter next(pfmbp->GetOutputBufPtr());
   S4PFObject *pfp = 0;
   while ((pfp = static_cast<S4PFObject *>(next()))) {
      S4HYBTrack *tp = pfp->GetHYBTrackPtr();
      if (!tp) continue;
      Int_t       j4mtrkid = tp->GetMotherTrackID();
      S4HYBTrack *mtrkp    = htmbp->GetHYBTrackPtr(j4mtrkid); 
      if (mtrkp) {
#if 1
         mtrkp->SetBit(BIT(16)); // flag as a kink mother
#endif
	 if (!scheme) {
            tp ->Lock(); // lock if kink daughter
            pfp->Lock(); // lock if kink daughter
	 } else {
            mtrkp->Lock(); // lock if kink mothter
	 }
      }
   }
   if (scheme) { // kink daughter scheme
      next.Reset();
      while ((pfp = static_cast<S4PFObject *>(next()))) {
         S4HYBTrack *tp = pfp->GetHYBTrackPtr();
         if (!tp) continue;
	 if (tp->TestBit(BIT(16))) pfp->Lock();
      }
   }
}
