//*************************************************************************
//* --------------------
//* S4VTXTrackMakerBuf
//* --------------------
//* (Description)
//*	This set of classes makes pad Tracks.
//* (Requires)
//* (Provides)
//* (Update Record)
//*	2004/08/28  A.Yamaguchi	Original version.
//*************************************************************************

#include <iostream>
#include "S4VTXTrackMakerBuf.h"
#include "S4VTXExactHit.h"
#include "S4VTXTrack.h"
#include "S4TPCTrack.h"
#include "S4TPCTrackHit.h"
#include "S4VTXTrackHit.h"
using namespace std;

//#define __DEBUG__

ClassImp(S4VTXTrackMakerBuf)

//=====================================================================
//* constructor -------------------------------------------------------
S4VTXTrackMakerBuf::S4VTXTrackMakerBuf(const Char_t    *name,
                                       const Char_t    *title, 
                                       S4VTXTrackMaker *module,
                                       Int_t            bufsize)
                  : U4VTXTrackMakerBuf(name, title, module, bufsize)
{
}
       
S4VTXTrackMakerBuf::S4VTXTrackMakerBuf(S4VTXTrackMaker *module,
                                       const Char_t    *name,
                                       const Char_t    *title, 
                                       Int_t            bufsize)
                  : U4VTXTrackMakerBuf(module, name, title, bufsize)
{
#ifdef __DEBUG__
   cerr << "constructor for S4VTXTrackMakerBuf is called..." << endl;
#endif
}

//=====================================================================
//* destructor --------------------------------------------------------

S4VTXTrackMakerBuf::~S4VTXTrackMakerBuf()
{
}

//=====================================================================
//* MakeOneTrack ----------------------------------------------------------

U4VTrack* S4VTXTrackMakerBuf::MakeOneTrack(U4VTrack *htkp)
{
   if (!IsInitialized()) {
      cerr << "ERROR  S4VTXTrackMakerBuf::MakeTracks"    << endl
           << "...... hit data not yet initialized." << endl;
      abort();
   }

   TObjArray *inputbufp = dynamic_cast<S4VTXHitMakerBuf *>
                          (GetInputDataBufPtr())->GetOutputBufPtr();
   S4VTXHit::SetSortBy("Time");
   inputbufp->UnSort();
   inputbufp->Sort();

   S4VTXTrack *trackp = new S4VTXTrack(static_cast<S4VTXTrackMaker *>(Module()));
   trackp->SetSeedTrackPtr(htkp);
   Int_t      hybID  = dynamic_cast<S4VTrack *>(htkp)->GetTrackID();

   TIter next(inputbufp);
   S4VTXHit *hitp = 0;
   while ((hitp = static_cast<S4VTXHit *>(next()))) {
      Int_t vtxID = hitp->GetExactHitPtr()->GetTrackID();
      if (vtxID == hybID) {
         S4VTXTrackHit *thitp = new S4VTXTrackHit(hitp);
         trackp->Add(thitp);
         if (hitp->IsLocked()) thitp->Lock();
      }
      if (trackp->GetTrackMakerPtr()->GetDetectorPtr()->GetNmeasPoints()
       == trackp->GetEntries()) break;
   }

   if (!trackp->Verify()) {
      delete trackp;
      return 0;
   }

   GetOutputBufPtr()->Add(trackp);
   return trackp;

}

//=====================================================================
//* //DebugPrint --------------------------------------------------------

void S4VTXTrackMakerBuf::DebugPrint(const Char_t * /*opt*/)
{
}
