{
//*----------------------------------------------------------------------------
//* ================
//*  Macro plotsgx.C
//* ================
//*
//* (Description)
//*	A sample macro to display plots created by cdcanl.C.
//* (Usage)
//*	$ jsf -b -q unpack.C	# unpack rawdata and produce runxxx.root.
//*	$ jsf -b -q cdcanl.C	# to analyze runxxx.root.
//*	$ jsf jsf.root		# start jsf in interactive mode
//*	......
//*	root [1] .x ploteffsg.C
//*
//*	This will display residual plots and opens an additional empty
//*	window as a temporary work canvas.
//*
//* (Update Record)
//*	2000/06/16	K.Fujii		Original version.
//*
//*----------------------------------------------------------------------------

gROOT->Reset();

gSystem->Load("libS4Utils.so");

Int_t nZoneX = 5;	// corresponds to # wires
Int_t nZoneY = 4;	// corresponds to # plots/wire

TCanvas *cs = new TCanvas("cs","DX-X",20,20,200*nZoneX,200*nZoneY);
cs->SetHighLightColor(5);
cs->SetFillColor(19);
cs->Divide(nZoneX,nZoneY);

TCanvas *c2 = new TCanvas("c2","Work",20,50+200*nZoneY,480,380);

const Int_t ncmds = 6;
Char_t *template[] = {
	"cs->cd(id++);\
	TH2D *h2 = hDXvsRes%2.2d;\
	h2->SetMarkerColor(46);\
	h2->Draw();\n",

	"Rebin(hDXvsRes%2.2d,5,4)->FitSlicesY();\n",

	"cs->cd(id++);\
	TH1D *hdx  = hDXvsRes%2.2d_chbn_2;\
	TH1D *hdx2 = new TH1D((*hdx)*(*hdx));\
	hdx2->SetName(\"Dx2%2.2d\");\
	hdx2->SetMarkerColor(46);\
	hdx2->SetMaximum(0.001);\
	hdx2->Draw();\n",

	"cs->cd(id++);\
	TH1D *h1 = Rebin(hDXvsSgt2%2.2d,5,1)->ProfileX()->ProjectionX();\
	h1->SetMarkerColor(46);\
	h1->SetMaximum(0.0004);\
	h1->Draw();\n",

	"TH1D *hdx2 = Dx2%2.2d;\
	TH1D *hsgt2 = hDXvsSgt2%2.2d_chbn_pfx_px;\
	TH1D *hsgx2 = new TH1D( (*hdx2) - (*hsgt2) );\
	hsgx2->SetName(\"Sgx2%2.2d\");\n",
	
	"cs->cd(id++);\
	TH1D *hsgx2 = Sgx2%2.2d;\
	TH1D *hsgx  = new TH1D(*hsgx2);\
	Sqrt(hsgx);\
	hsgx->SetName(\"Sgx%2.2d\");\
	hsgx->SetMarkerColor(46);\
	hsgx->SetMinimum(0.);\
	hsgx->SetMaximum(0.04);\
	hsgx->Draw();\n"
};

gStyle->SetOptFit();	// to display fit parameters in each window

Int_t cellid = 1;
Int_t id = 1;	// directory counter
Int_t j;
for (j = 0; j < ncmds; j++) {
	Int_t i;
	for (i = 0; i < nZoneX; i++) {
		Char_t command[2048];
		Int_t a = 5*cellid + i;
		sprintf(command,template[j],a,a,a);
		cerr << command << endl;
		gROOT->ProcessLine(command);
		cs->Update();
	}
}
c2->cd();
}


