{
//*----------------------------------------------------------------------------
//* ================
//*  Macro ploteff.C
//* ================
//*
//* (Description)
//*	A sample macro to display plots created by cdcanl.C.
//* (Usage)
//*	$ jsf -b -q unpack.C	# unpack rawdata and produce runxxx.root.
//*	$ jsf -b -q cdcanl.C	# to analyze runxxx.root.
//*	$ jsf jsf.root		# start jsf in interactive mode
//*	......
//*	root [1] .x ploteff.C
//*
//*	This will display residual plots and opens an additional empty
//*	window as a temporary work canvas.
//*
//* (Update Record)
//*	2000/06/16	K.Fujii		Original version.
//*
//*----------------------------------------------------------------------------

gROOT->Reset();

if( strncmp(gSystem->HostName(),"ccjlc",5)  != 0 ) {
    	if( strncmp(gSystem->Getenv("OSTYPE"),"hpux",4) == 0 ) {
      		gSystem->Load("libCDCUtils.sl");
	} else {
      		gSystem->Load("libCDCUtils.so");
    	}
}

Int_t nZoneX = 5;	// corresponds to # wires
Int_t nZoneY = 4;	// corresponds to # plots/wire

TCanvas *ce = new TCanvas("ce","DX-X",20,20,200*nZoneX,200*nZoneY);
ce->SetHighLightColor(5);
ce->SetFillColor(19);
ce->Divide(nZoneX,nZoneY);

TCanvas *c2 = new TCanvas("c2","Work",20,50+200*nZoneY,480,380);

const Int_t ncmds = 4;
Char_t *template[] = {
	"ce->cd(id++);\
	TH2F *h2 = PhPk%2.2d;\
	h2->SetMarkerColor(46);\
	h2->Draw();\n",
	
	"ce->cd(id++);\
	TH2F *h2 = Ph%2.2d;\
	h2->SetMarkerColor(46);\
	h2->Draw();\n",
	
	"ce->cd(id++);\
	TH2F *h2 = Res%2.2d;\
	h2->SetMarkerColor(46);\
	h2->Draw();\n",

	"ce->cd(id++);\
	TH1E *he = Eff%2.2d;\
	he->SetMarkerColor(46);\
	he->SetMinimum(0.6);\
	he->SetMaximum(1.1);\
	he->Draw();\n"
};

gStyle->SetOptFit();	// to display fit parameters in each window

Int_t id = 1;	// directory counter
Int_t j;
for (j = 0; j < ncmds; j++) {
	Int_t i;
	for (i = 0; i < nZoneX; i++) {
		Char_t command[256];
		sprintf(command,template[j],i,i);
		gROOT->ProcessLine(command);
		ce->Update();
	}
}
c2->cd();
}


