#ifndef __USERANALYSIS__
#define __USERANALYSIS__

#include "VUserAnalysis.h"
#include "TStyle.h"
#include "TH1.h"
#include "TH2.h"
#include "TGraph.h"
#include "TMath.h"
#include "TPaveLabel.h"
#include "U4BabyTrackMaker.h"
#include "U4BabyTrack.h"
#include "U4CDCHit.h"

#include "HardwareMapParam.dat"


class UserAnalysis : public VUserAnalysis {
public:
	UserAnalysis(BabyDisplay *display, U4CDCFADCCalibration *calib);
	virtual ~UserAnalysis();

	void InitRawFADC();
	void DrawRawFADC();
	void InitCounters();
	void DrawCounters();
	void InitScaler();
	void DrawScaler();
	void UpdateCounterInfo();
	void InitDriftLengthVs();
	void DrawDriftLengthVs();
	
	Double_t GetGPIB();
	Bool_t   CheckScalerHits(Int_t &nhits); // (0,1)= (Between Spills, During Spill),
						// returns No of hole hodoscope hits and
						// fills fHodoX & fHodoY array.

	void	 SetSpillThreshold(Int_t th = 5000) { fSpillThreshold = th; }
	
	void Initialize();
	void DrawHist();

private:
	Int_t	     fSpillThreshold;	//! Threshold of "During SPILL"
	Double_t     fHodoX[__HODOX__]; //!
	Double_t     fHodoY[__HODOY__]; //! 
        TPaveLabel   fLabel;		//!
	TH2F        *hTOFADCTOF1;	//!
	TH2F        *hTOFADCTOF2;	//!
	TH2F        *hADCTOF12;		//!
	TH2F        *hTC1TC2;		//!
	TH2F        *hCH1CH2;		//!
	TH2F        *hCH1PBG;		//!
	TH1F       **hFADC;		//!
	TH2F       **hTPK;		//!
	TH2F       **hTQ;		//!
	TH2F       **hTOFTQ;		//!
	TH2F	   **hHHvsDL;		//! HodoscopeHit vs DriftLength
	TH2F	   **hIvsDL;		//! Current vs DriftLength
	TH2F	   **hEVvsXSbs;		//! EventNo vs X at Sheelding wire between spills
	TH2F	   **hEVvsXSds;		//! EventNo vs X at Sheelding wire during spill
	TH1F	    *hHODOX;		//! Hodoscope X-axis Hits
	TH1F	    *hHODOY;		//! Hodoscope Y-axis Hits
	TH2F	    *hHODO2;		//! Hodoscope XY scatter plot
	
	ClassDef(UserAnalysis, 1)   // A class to analyze event
};

#endif


