//*************************************************************************
//* --------------------
//* U4CDCHitMaker
//* --------------------
//* (Description)
//*	This set of classes makes pad hits.
//* (Requires)
//* (Provides)
//* (Update Record)
//*	2001/08/22  K.Hoshina	Original version.
//*************************************************************************

#include "U4CDCHitMaker.h"
#include "U4CDCHitMakerBuf.h"
#include "U4CDCHitWire.h"

ClassImp(U4CDCHitMaker)

//=====================================================================
//* constructor -------------------------------------------------------

U4CDCHitMaker::U4CDCHitMaker(U4CDCTimeToXRelation *calib,
			       const char *name, const char *title)
       : U4VHitMaker(name,title), 
         fToDeleteCalib(calib ? kFALSE : kTRUE), fCalibPtr(calib) 
{
}
         
//=====================================================================
//* destructor --------------------------------------------------------

U4CDCHitMaker::~U4CDCHitMaker()
{
	if (fToDeleteCalib && fCalibPtr) delete fCalibPtr;
}

//=====================================================================
//* SetCalibrationData ------------------------------------------------

void U4CDCHitMaker::SetCalibrationData(U4CDCTimeToXRelation *calib)
{
	fCalibPtr = calib;
	fToDeleteCalib = kFALSE;
	((U4CDCHitMakerBuf *)EventBuf())->SetCalibrationData(calib);
}

//=====================================================================
//* Initialize() ------------------------------------------------------

Bool_t U4CDCHitMaker::Initialize()
{
	if (!fCalibPtr) {
		fCalibPtr = new U4CDCTimeToXRelation();
		fToDeleteCalib = kTRUE;
	}
	fEventBuf = new U4CDCHitMakerBuf(this,fCalibPtr);
	
	return kTRUE;
}

//=====================================================================
//* BeginRun ----------------------------------------------------------


Bool_t U4CDCHitMaker::BeginRun(Int_t run)
{

	fCalibPtr->Update();

	return kTRUE;

}

//=====================================================================
//* Process -----------------------------------------------------------

Bool_t U4CDCHitMaker::Process(Int_t ev)
{
	U4CDCClusterMaker *clmp
		= (U4CDCClusterMaker*)gJSF->FindModule("U4CDCClusterMaker");
	U4CDCClusterBuf *clbufp = (U4CDCClusterBuf *)clmp->EventBuf();

	U4CDCHitMakerBuf  *phbufp = (U4CDCHitMakerBuf *)EventBuf();
	phbufp->SetInputDataBufPtr(clbufp);	
	phbufp->MakeHits();

	return kTRUE;
}







