//*************************************************************************
//* ---------------------
//* U4CDCTrackHit Class
//* ---------------------
//* (Description)
//*	Class for a single Baby hit.
//* (Requires)
//*	TObject
//* (Provides)
//*	U4CDCTrackHit		: 
//* (Update Record)
//*	2002/08/28  R.Kuboshima
//*     2004/09/05  K.Fujii        Now inherits from U4VTrack.
//*************************************************************************

#include "U4CDCTrackHit.h"

ClassImp(U4CDCTrackHit)

//=====================================================================
//* constructor -------------------------------------------------------


U4CDCTrackHit::U4CDCTrackHit(U4CDCHit *htp, Int_t lr)
              : U4VTrackHit(htp),
                fLeftRight(lr),
                fVHitPtr(0)
{
#if 0
		std::cerr << " U4CDCTrackHit::constructor "
			  << " fHitPtr = " << (void *)fHitPtr
			  << " TrackHit = " << (void *)this << std::endl;
#endif
}
	
U4CDCTrackHit::U4CDCTrackHit(const U4CDCTrackHit &hit)
              : U4VTrackHit(hit),
                fLeftRight(hit.fLeftRight),
                fVHitPtr(hit.fVHitPtr)
{
}

//=====================================================================
//* destructor --------------------------------------------------------

U4CDCTrackHit::~U4CDCTrackHit()
{
}

//=====================================================================
//* SetSortBy --------------------------------------------------------
                                                                                
void U4CDCTrackHit::SetSortBy(Char_t *opt)
{
   switch (opt[1]) {
      case 'h':       // sort by "Chi2"
         U4VTrackHit::SetSortBy(0);
         break;
      case '0':       // sort by "T0"
         U4VTrackHit::SetSortBy(1);
         break;
      case 'i':       // sort by "WireLayer"
      default:        // sort by "WireLayer"
         U4VTrackHit::SetSortBy(5);
         break;
   }
}
                                                                                
//=====================================================================
//* IsSoatable --------------------------------------------------------
                                                                                
Bool_t U4CDCTrackHit::IsSortable() const
{
   return kTRUE;
}
                                                                                
//=====================================================================
//* Compare -----------------------------------------------------------

Int_t U4CDCTrackHit::Compare(const TObject *obj) const
{
   switch (GetSortBy()) {
      case 0: { // Sort by Chi2
         Double_t res  = TMath::Abs(GetResidual());
         Double_t reso = TMath::Abs(((U4VTrackHit *)obj)->GetResidual());
         return (res < reso ? -1 : (res > reso ? +1 : 0));
      }
      case 1: { // Sort by T0
         Double_t t0  = GetT0();
         Double_t t0o = ((U4VTrackHit *)obj)->GetT0();
         return (t0 < t0o ? -1 : (t0 > t0o ? +1 : 0));
      }
      case 5: // layer serial number
      default: {
         Int_t layer  = ((U4CDCTrackHit *)this)->GetHitPtr()->GetWirePtr()->GetDriftRegionNumber();
         Int_t layero = ((U4CDCTrackHit *)obj)->GetHitPtr()->GetWirePtr()->GetDriftRegionNumber();
         return (layer < layero ? -1 : (layer > layero ? +1 : 0));
      }
   }
}
