#ifndef __U4CDCVECTORHIT__
#define __U4CDCVECTORHIT__

//*************************************************************************
//* ---------------------
//* U4CDCVectorHit Class
//* ---------------------
//* (Description)
//*	Class to store "U4CDCHitWire"s for a single cell.
//* (Requires)
//*	TObjArray
//* (Provides)
//*	U4CDCVectorHit		: Array containing "U4CDCHitWire"s
//* (Update Record)
//*	2002/11/15  R.Kuboshima     Original Ver.
//*************************************************************************

class U4CDCTrack;
class U4CDCTrackHit;

#include "U4CDCTrackHit.h"
#include "U4CDCTrack.h"

class U4CDCVectorHit : public TObjArray, public TAttLockable {
friend class U4CDCTrack;

public:
  U4CDCVectorHit();
  U4CDCVectorHit(const U4CDCVectorHit &vh, Bool_t mirror = kFALSE);
  virtual ~U4CDCVectorHit();

  inline virtual U4CDCTrack  *GetTrackPtr   () const { return fTrackPtr;    }
  inline virtual Int_t	      GetLayerNumber() const { return fLayerNumber; }
  inline virtual Int_t	      GetCellNumber () const { return fCellNumber;  }
  inline virtual Int_t	      GetCellID     () const { return fCellID;      }
  inline virtual Int_t        GetTrackType  ();
  inline virtual Double_t     GetChi2       ();
  inline virtual Double_t     Xat           (Double_t y);
  inline virtual Double_t     SGX2at        (Double_t y); 
  inline virtual Double_t     GetSlope      ();
  inline virtual Double_t     GetIntercept  ();

  inline virtual void     SetTrackType  (Int_t type)      { fTrackType   = type; }
  inline virtual void     SetTrackPtr   (U4CDCTrack *tkp) { fTrackPtr    = tkp;  }
  inline virtual void     SetLayerNumber(Int_t i)         { fLayerNumber = i;    }
  inline virtual void     SetCellNumber (Int_t i)         { fCellNumber  = i;    }
  inline virtual void     SetCellID     (Int_t i)         { fCellID      = i;    }

  inline virtual Bool_t  IsSortable()                const { return kTRUE;     }
         virtual Int_t	 Compare(const TObject *obj) const;
         virtual void    Add    (TObject *obj);
         virtual void    Mirror ();

         virtual void    Fit   ();
  inline virtual void    Update() 
  { 
	  fDone = kTRUE;    
	  Fit(); 
  }
         virtual void 	 DebugPrint(const Char_t *opt = "Brief");

private:
  Int_t		 fLayerNumber;		 // layer Number
  Int_t		 fCellNumber;		 // cell Number 
  Int_t		 fCellID;		 // cell id
  U4CDCTrack    *fTrackPtr;		 // pointer to track this belongs to
  Bool_t         fDone;                  // (1,0) = (fit done, not yet)
  Int_t          fTrackType;             // track type
  Double_t       fChi2;                  // chi^2
  Double_t       fa;                     // a when x = a*y + b
  Double_t       fb;                     // b
  Double_t       fEaa;                   // <da*da>
  Double_t       fEba;                   // <db*da>
  Double_t       fEbb;                   // <db*db>


  ClassDef(U4CDCVectorHit, 1) 		// Baby VectorHit

};

//-----------------------------------------------------------
//  inline function
//-----------------------------------------------------------

Int_t    U4CDCVectorHit::GetTrackType()
{
   if (!fDone) Update();
   return fTrackType;
}
Double_t U4CDCVectorHit::GetChi2()
{
   if (!fDone) Update();
   return fChi2;
}
Double_t U4CDCVectorHit::Xat(Double_t y)
{
   if (!fDone) Update();
   return (fa*y + fb);
}
Double_t U4CDCVectorHit::SGX2at(Double_t y)
{ 
   if (!fDone) Update();    
   return (fEaa*y*y + 2*fEba*y + fEbb);
}
Double_t U4CDCVectorHit::GetSlope()
{
   if (!fDone) Update();
   return fa;
}
Double_t U4CDCVectorHit::GetIntercept()
{
   if (!fDone) Update();
   return fb;
}

#endif
