//*************************************************************************
//* --------------------
//* U4CDCRawScalerDatum Class
//* --------------------
//* (Description)
//* 	Classes for a single Scaler datum.
//* (Requires)
//*	TObject
//* (Provides)
//*	U4CDCRawScalerDatum
//* (Update Record)
//*	1999/06/02  K.Fujii	Original version.
//*
//*************************************************************************


#include "JSFSteer.h"
#include "U4CDCRawScalerDatum.h"
#include <iostream>

ClassImp(U4CDCRawScalerDatum)

// static variables are not declared as constant so as to allow
// different formats in future.

Int_t U4CDCRawScalerDatum::gFormatID  =  0;	// Format ID
Int_t U4CDCRawScalerDatum::gNstabits  =  0;	// No. station number bits
Int_t U4CDCRawScalerDatum::gNchabits  =  4;	// No. channel number bits
Int_t U4CDCRawScalerDatum::gNswperdt  =  3;	// No. short words per Scaler datum
Int_t U4CDCRawScalerDatum::gNchMax    = 12;	// Maximum Scaler channel No

//___________________________________________________________________________
// ------------
// Constructors
// ------------
//
U4CDCRawScalerDatum::U4CDCRawScalerDatum() 
	: fStationNo(0), fChannelNo(0), fScaler(0)  {}
//___________________________________________________________________________
U4CDCRawScalerDatum::U4CDCRawScalerDatum(U4CDCRawScalerDatum & rad) 
{
    fStationNo 	= rad.fStationNo;
    fChannelNo 	= rad.fChannelNo;
    fScaler	= rad.fScaler;
}

//___________________________________________________________________________
U4CDCRawScalerDatum::U4CDCRawScalerDatum(SCLR_t sclrword[])
{
    Unpack(sclrword);
}

//___________________________________________________________________________
// ------------
// Destructors
// ------------
//
U4CDCRawScalerDatum::~U4CDCRawScalerDatum() {}

//___________________________________________________________________________
// -------
// Setters
// -------
//
// This methods should be modified according to your raw data format.
//
void U4CDCRawScalerDatum::SetFormat(U4CDCUniversalRecordHeader & urh)
{ 
    gFormatID = urh.GetFormatID();
    // set gNstabits, etc., according to the format ID.
    switch (gFormatID) {
        case 2: {
            gNstabits =  5;
            gNchabits =  4;
            gNchMax   = 12;
            break;
        }
        case 0: 
        case 1: 
        default: { 
            std::cerr << "U4CDCRawScalerDatum::Unpack : Format ID = " << gFormatID
                 << " is not supported" << std::endl;
	}
    }
}

//___________________________________________________________________________
// ---------------
// Service Methods
// ---------------
//
// This methods should be modified according to your raw data format.
//
void U4CDCRawScalerDatum::Unpack(SCLR_t sclrword[])
{
    switch (gFormatID) {
  	case 2: {
	   union { 
		UInt_t ui;
		UShort_t usi[2];
	   } tmpunion;
#ifndef __LITTLE_ENDIAN__
	   for (Int_t i=0; i<3; i++) {
           	sclrword[i] = (sclrword[i] << 8) | (sclrword[i] >> 8);
	   }
	   for (Int_t i=0; i<2; i++) tmpunion.usi[i] = sclrword[i+1];
#else
	   for (Int_t i=0; i<2; i++) tmpunion.usi[i] = sclrword[2-i];
#endif
  	   fStationNo = sclrword[0] >> (sizeof(SCLR_t)*8 - gNstabits);
  	   SCLR_t tmp = sclrword[0] << (sizeof(SCLR_t)*8 - gNchabits);
  	   fChannelNo = tmp  >> (sizeof(SCLR_t)*8 - gNchabits);
	   // tmpunion.usi[0]     = (tmpunion.usi[0] << 8) >> 8; 
  	   fScaler    = (Int_t)tmpunion.ui; 
#ifdef __DEBUG__
		std::cerr << "N,A,ScalerVal = " << fStationNo << " "
		     << fChannelNo << " " << fScaler << std::endl;
		std::cerr << "word1 = " << tmpunion.usi[0] << std::endl;
		std::cerr << "word2 = " << tmpunion.usi[1] << std::endl;
#endif
	   break;
  	}
    	case 0: 
  	case 1:
        default: { 
            std::cerr << "U4CDCRawScalerDatum::Unpack : Format ID = " << gFormatID
                 << " is not supported" << std::endl;
        }
    }
}

//___________________________________________________________________________
void U4CDCRawScalerDatum::DebugPrint(const Char_t *)
{
    std::cerr << " n " << fStationNo
         << " a " << fChannelNo
         << " val " << fScaler << std::endl;
}
