//*************************************************************************
//* ------------------------------
//* U4CDCUniversalRecordHeader Class
//* ------------------------------
//* (Description)
//*	This class contains the header information that all the
//*	U4CDC raw records carry in common.
//* (Requires)
//*	TObject
//* (Provides)
//*	U4CDCUniversalRecordHeader	: universal record header
//* (Update Record)
//*	1999/06/02  K.Fujii	Original version.
//*
//*************************************************************************

#include "JSFSteer.h"
#include "U4CDCUniversalRecordHeader.h"
#include <iostream>

ClassImp(U4CDCUniversalRecordHeader)

//
// gNElms: No. elements in URH excluding the 1st "NWORDS" word.
//
Len_t U4CDCUniversalRecordHeader::gNElms     = 4;
Len_t U4CDCUniversalRecordHeader::gLElms     = gNElms * sizeof(URH_t)/2;
Len_t U4CDCUniversalRecordHeader::gURHLength = gLElms + sizeof(Len_t)/2;

//___________________________________________________________________________
U4CDCUniversalRecordHeader::U4CDCUniversalRecordHeader() 
  	: fBufferLength(0), fFormatID(0), fRecordType(0), 
  	  fRunNumber(0), fEventNumber(0)
{
	if (gNElms > 3) fFormatID = 1;
}

//___________________________________________________________________________
U4CDCUniversalRecordHeader::U4CDCUniversalRecordHeader(const U4CDCUniversalRecordHeader & h) 
  	: fBufferLength(h.fBufferLength), fFormatID(h.fFormatID), 
	  fRecordType(h.fRecordType), 
  	  fRunNumber(h.fRunNumber), fEventNumber(h.fEventNumber)
{
}

//___________________________________________________________________________
const U4CDCUniversalRecordHeader& 
U4CDCUniversalRecordHeader::operator=(const U4CDCUniversalRecordHeader & h) 
{
	fBufferLength = h.fBufferLength;
	fFormatID     = h.fFormatID;
	fRecordType   = h.fRecordType;
	fRunNumber    = h.fRunNumber;
	fEventNumber  = h.fEventNumber;
	return *this;
}
//___________________________________________________________________________
U4CDCUniversalRecordHeader::~U4CDCUniversalRecordHeader() {}


//___________________________________________________________________________
Bool_t U4CDCUniversalRecordHeader::ReadFile(FILE *fd) 
{
#ifndef __LITTLE_ENDIAN__
    UShort_t dum[2];
    Len_t ne = fread(dum, sizeof(Len_t), 1, fd);
    dum[0] = (dum[0] << 8) | (dum[0] >> 8);
    dum[1] = (dum[1] << 8) | (dum[1] >> 8);
    UInt_t dum4 = dum[1];
    fBufferLength = (dum4 << 16) | dum[0];
#else
    Len_t ne = fread(&fBufferLength, sizeof(Len_t), 1, fd);
#endif
#if 0 
std::cerr << " ns:fd = " << (void*)fd << std::endl;
std::cerr << " ns:sizeof(Len_t) = " << sizeof(Len_t) << std::endl;
std::cerr << " ns:ne, fBufferLength = " << ne << " " << fBufferLength << std::endl;
#endif

    if ( ne != 1 || fBufferLength <= 0 ) {
        Error("U4CDCUniversalRecordHeader","Failed to read Buffer Length.");
	std::cerr << " ne, fBufferLength = " << ne << " " << fBufferLength << std::endl;
        fRecordType = -9999;
        return kFALSE;
    }
    URH_t *hd = new URH_t[gNElms];
    ne = fread(hd, sizeof(URH_t), gNElms, fd);
    if ( ne != gNElms ) {
       Error("U4CDCUniversalRecordHeader","Failed to read URH.");
       delete hd;
       return kFALSE;
    }
    Unpack(hd);
    delete hd;
    return kTRUE;
}

//___________________________________________________________________________
void U4CDCUniversalRecordHeader::Unpack(URH_t hd[])
{
#ifndef __LITTLE_ENDIAN__
    for (Int_t i = 0; i < gLElms; i++) {
        hd[i] = (hd[i] << 8) | (hd[i] >> 8);
    }
#endif
    switch (fFormatID) {
    	case 0: { 
            fRunNumber   = hd[0];
            fEventNumber = hd[1];
            fRecordType  = (SRH_t)hd[2];
            
            break;
        } 
    	case 1: 
    	case 2: { 
            fFormatID    = hd[0];
            fRecordType  = hd[1];
            fRunNumber   = hd[2];
            fEventNumber = hd[3];
            
            break;
        }
        default: {
            std::cerr << "Format ID = " << fFormatID
                 << " is not supported" << std::endl;
        }
    }
    if (fRecordType == kRecTypeBRR) gJSF->SetRunNumber(fRunNumber);
    gJSF->SetEventNumber(fEventNumber);
}

//___________________________________________________________________________
void U4CDCUniversalRecordHeader::DebugPrint(const Char_t *)
{
    std::cerr << " No. words = "  << fBufferLength
         << " Format = "     << fFormatID
         << " Record ID = "  << fRecordType
         << " Run = "        << fRunNumber
         << " Event = "      << fEventNumber << std::endl;
}

