#ifndef __U4CDCUNIVERSALRECORDHEADER__
#define __U4CDCUNIVERSALRECORDHEADER__

//*************************************************************************
//* ------------------------------
//* U4CDCUniversalRecordHeader Class
//* ------------------------------
//* (Description)
//*	This class contains the header information that all the
//*	U4CDC raw records carry in common.
//* (Requires)
//*	TObject
//* (Provides)
//*	U4CDCUniversalRecordHeader	: universal record header
//* (Update Record)
//*	1999/06/02  K.Fujii	Original version.
//*************************************************************************

#include <iostream>
#include "TObject.h"
#include "TDatime.h"

#include "JSFModule.h"

#include "U4VTypeDef.h"

// -----------------------
// Universal Record Header
// -----------------------

static  const Int_t kRecTypeField  =  4;
static  const Int_t kRecTypeBRR    =  1;
static  const Int_t kRecTypeERR    =  2;
static  const Int_t kRecTypeADC    = 10;
static  const Int_t kRecTypeGPIB   = 11;
static  const Int_t kRecTypeScaler = 12;

class U4CDCUniversalRecordHeader : public TObject {
private:
  Len_t   fBufferLength;	//  Buffer length in units of 2byes
  URH_t   fFormatID;		//  Format ID
  SRH_t   fRecordType;		//  Record type
  URH_t   fRunNumber;		//  Run number
  URH_t   fEventNumber;		//  Event number
  static  Len_t gNElms;		//! No. of elements in URH
  static  Len_t gLElms;		//! No. of words for elements in URH
  static  Len_t gURHLength;	//! No. of words for URH

public:
  U4CDCUniversalRecordHeader();
  U4CDCUniversalRecordHeader(const U4CDCUniversalRecordHeader& h);
  virtual ~U4CDCUniversalRecordHeader();
 
  const U4CDCUniversalRecordHeader& operator=(const U4CDCUniversalRecordHeader& h);
  
  Bool_t	ReadFile(FILE *fd);
  void		Unpack(URH_t hd[]);
  void		DebugPrint(const Char_t *opt = "");

  inline Len_t  GetBufferLength() const { return fBufferLength;              }
  inline Len_t  GetDataLength  () const { return (fBufferLength-gURHLength); }
  inline Int_t  GetFormatID    () const { return fFormatID;                  }
  inline Int_t  GetRecordType  () const { return fRecordType;                }
  inline Int_t  GetRunNumber   () const { return fRunNumber;                 }
  inline Int_t  GetEventNumber () const { return fEventNumber;               }

  inline static void UseOldFormat()     {
	std::cerr << ">>>>>>>>> U4CDCUniversalRecordHeader::UseOldFormat():" << std::endl
	     << "   Going to use 1998 format " << std::endl;
	gNElms = 3; 
	gLElms     = gNElms * sizeof(URH_t)/2;
	gURHLength = gLElms + sizeof(Len_t)/2;
  }

  ClassDef(U4CDCUniversalRecordHeader,2) // U4CDC Universal Record Header
};

#endif

