//*************************************************************************
//* ---------------------
//* U4TPCKalHit Class
//* ---------------------
//* (Description)
//* (Requires)
//*     TObjArray
//* (Provides)
//* (Update Record)
//*     2004/08/26  A.Yamaguchi    Original Ver.
//*************************************************************************

#include <iostream>
#include <iomanip>

#include "U4TPCKalHit.h"
#if ROOT_VERSION_CODE >= ROOT_VERSION(5,15,0)
#include "TMath.h"
#endif

ClassImp(U4TPCKalHit)

//=====================================================================
//* constructor -------------------------------------------------------

U4TPCKalHit::U4TPCKalHit(Int_t m)
           : U4VKalHit(m),
             fSide(0),
             fVdrift(0.7e-3)
{
}
      
U4TPCKalHit::U4TPCKalHit(const TObject        &hit,
                         const U4TPCMeasLayer &ms,
                         Double_t             *x,
                         Double_t             *dx, 
                         Int_t                 side,
                         Double_t              v,
                         Double_t              b,
                         Int_t                 m)
           : U4VKalHit(hit, ms, x, dx, b, m),
             fSide(side), 
             fVdrift(v)
{
}

U4TPCKalHit::U4TPCKalHit(const U4TPCKalHit &hit)
           : U4VKalHit(hit),
             fSide(hit.fSide),
             fVdrift(hit.fVdrift)
{
}

//=====================================================================
//* destructor -------------------------------------------------------

U4TPCKalHit::~U4TPCKalHit()
{
}

//=====================================================================
//* XvToMv ------------------------------------------------------------

TKalMatrix U4TPCKalHit::XvToMv(const TVector3 &xv, Double_t t0) const
{
   const U4TPCMeasLayer &ms
                 = dynamic_cast<const U4TPCMeasLayer &>(GetMeasLayer());
   TKalMatrix h  = ms.XvToMv(xv,GetSide());
   Double_t r    = ms.GetR();
   Double_t phih = h(0,0) / r;
   Double_t phim = (*this)(0,0) / r;
   Double_t dphi = phih - phim;

   static Double_t kPi    = TMath::Pi();
   static Double_t kTwoPi = 2 * kPi;

   while (dphi < -kPi) dphi += kTwoPi;
   while (dphi >  kPi) dphi -= kTwoPi;

   h(0,0)  = r * (phim + dphi);
   h(1,0) += fVdrift * t0;       // T0 shift

   return h;
}

//=====================================================================
//* DebugPrint --------------------------------------------------------

void U4TPCKalHit::DebugPrint(Option_t *) const
{
   using namespace std;
   cerr << "------------------- Site Info -------------------------" << endl;

   for (Int_t i=0; i<GetDimension(); i++) {
      Double_t x  = (*this)(i,0);
      Double_t dx = (*this)(i,1);
      cerr << " x[" << i << "] = " << setw(8) << setprecision(5) << x
           << "    "
           << "dx[" << i << "] = " << setw(6) << setprecision(2) << dx
           << setprecision(7)
           << resetiosflags(ios::showpoint)
           << endl;
   }
   cerr << " r     = "
        << static_cast<const U4TPCMeasLayer&>(GetMeasLayer()).GetR() << endl;
   cerr << "-------------------------------------------------------" << endl;
}
